/*     */ package com.google.code.yanf4j.buffer;
/*     */ 
/*     */ import com.google.code.yanf4j.util.CircularQueue;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.nio.ByteOrder;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import java.util.Queue;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class CachedBufferAllocator
/*     */   implements IoBufferAllocator
/*     */ {
/*     */   private static final int DEFAULT_MAX_POOL_SIZE = 8;
/*     */   private static final int DEFAULT_MAX_CACHED_BUFFER_SIZE = 262144;
/*     */   private final int maxPoolSize;
/*     */   private final int maxCachedBufferSize;
/*     */   private final ThreadLocal<Map<Integer, Queue<CachedBuffer>>> heapBuffers;
/*     */   private final ThreadLocal<Map<Integer, Queue<CachedBuffer>>> directBuffers;
/*     */   
/*     */   public CachedBufferAllocator()
/*     */   {
/*  82 */     this(8, 262144);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public CachedBufferAllocator(int maxPoolSize, int maxCachedBufferSize)
/*     */   {
/*  98 */     if (maxPoolSize < 0) {
/*  99 */       throw new IllegalArgumentException("maxPoolSize: " + maxPoolSize);
/*     */     }
/* 101 */     if (maxCachedBufferSize < 0) {
/* 102 */       throw new IllegalArgumentException("maxCachedBufferSize: " + maxCachedBufferSize);
/*     */     }
/*     */     
/* 105 */     this.maxPoolSize = maxPoolSize;
/* 106 */     this.maxCachedBufferSize = maxCachedBufferSize;
/*     */     
/* 108 */     this.heapBuffers = new ThreadLocal()
/*     */     {
/*     */       protected Map<Integer, Queue<CachedBufferAllocator.CachedBuffer>> initialValue() {
/* 111 */         return CachedBufferAllocator.this.newPoolMap();
/*     */       }
/* 113 */     };
/* 114 */     this.directBuffers = new ThreadLocal()
/*     */     {
/*     */       protected Map<Integer, Queue<CachedBufferAllocator.CachedBuffer>> initialValue() {
/* 117 */         return CachedBufferAllocator.this.newPoolMap();
/*     */       }
/*     */     };
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int getMaxPoolSize()
/*     */   {
/* 128 */     return this.maxPoolSize;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int getMaxCachedBufferSize()
/*     */   {
/* 138 */     return this.maxCachedBufferSize;
/*     */   }
/*     */   
/*     */   private Map<Integer, Queue<CachedBuffer>> newPoolMap()
/*     */   {
/* 143 */     Map<Integer, Queue<CachedBuffer>> poolMap = new HashMap();
/* 144 */     int poolSize = this.maxPoolSize == 0 ? 8 : this.maxPoolSize;
/* 145 */     for (int i = 0; i < 31; i++) {
/* 146 */       poolMap.put(Integer.valueOf(1 << i), new CircularQueue(poolSize));
/*     */     }
/* 148 */     poolMap.put(Integer.valueOf(0), new CircularQueue(poolSize));
/* 149 */     poolMap.put(Integer.valueOf(Integer.MAX_VALUE), new CircularQueue(poolSize));
/* 150 */     return poolMap;
/*     */   }
/*     */   
/*     */   public IoBuffer allocate(int requestedCapacity, boolean direct)
/*     */   {
/* 155 */     int actualCapacity = IoBuffer.normalizeCapacity(requestedCapacity);
/*     */     IoBuffer buf;
/* 157 */     IoBuffer buf; if ((this.maxCachedBufferSize != 0) && (actualCapacity > this.maxCachedBufferSize)) { IoBuffer buf;
/* 158 */       if (direct) {
/* 159 */         buf = wrap(ByteBuffer.allocateDirect(actualCapacity));
/*     */       }
/*     */       else {
/* 162 */         buf = wrap(ByteBuffer.allocate(actualCapacity));
/*     */       }
/*     */     } else {
/*     */       Queue<CachedBuffer> pool;
/*     */       Queue<CachedBuffer> pool;
/* 167 */       if (direct) {
/* 168 */         pool = (Queue)((Map)this.directBuffers.get()).get(Integer.valueOf(actualCapacity));
/*     */       }
/*     */       else {
/* 171 */         pool = (Queue)((Map)this.heapBuffers.get()).get(Integer.valueOf(actualCapacity));
/*     */       }
/*     */       
/*     */ 
/* 175 */       buf = (IoBuffer)pool.poll();
/* 176 */       if (buf != null) {
/* 177 */         buf.clear();
/* 178 */         buf.setAutoExpand(false);
/* 179 */         buf.order(ByteOrder.BIG_ENDIAN);
/*     */ 
/*     */       }
/* 182 */       else if (direct) {
/* 183 */         buf = wrap(ByteBuffer.allocateDirect(actualCapacity));
/*     */       }
/*     */       else {
/* 186 */         buf = wrap(ByteBuffer.allocate(actualCapacity));
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 191 */     buf.limit(requestedCapacity);
/* 192 */     return buf;
/*     */   }
/*     */   
/*     */   public ByteBuffer allocateNioBuffer(int capacity, boolean direct)
/*     */   {
/* 197 */     return allocate(capacity, direct).buf();
/*     */   }
/*     */   
/*     */   public IoBuffer wrap(ByteBuffer nioBuffer)
/*     */   {
/* 202 */     return new CachedBuffer(nioBuffer);
/*     */   }
/*     */   
/*     */   public void dispose() {}
/*     */   
/*     */   private class CachedBuffer
/*     */     extends AbstractIoBuffer
/*     */   {
/*     */     private final Thread ownerThread;
/*     */     private ByteBuffer buf;
/*     */     
/*     */     protected CachedBuffer(ByteBuffer buf)
/*     */     {
/* 215 */       super(buf.capacity());
/* 216 */       this.ownerThread = Thread.currentThread();
/* 217 */       this.buf = buf;
/* 218 */       buf.order(ByteOrder.BIG_ENDIAN);
/*     */     }
/*     */     
/*     */     protected CachedBuffer(CachedBuffer parent, ByteBuffer buf)
/*     */     {
/* 223 */       super();
/* 224 */       this.ownerThread = Thread.currentThread();
/* 225 */       this.buf = buf;
/*     */     }
/*     */     
/*     */ 
/*     */     public ByteBuffer buf()
/*     */     {
/* 231 */       if (this.buf == null) {
/* 232 */         throw new IllegalStateException("Buffer has been freed already.");
/*     */       }
/* 234 */       return this.buf;
/*     */     }
/*     */     
/*     */ 
/*     */     protected void buf(ByteBuffer buf)
/*     */     {
/* 240 */       ByteBuffer oldBuf = this.buf;
/* 241 */       this.buf = buf;
/* 242 */       free(oldBuf);
/*     */     }
/*     */     
/*     */ 
/*     */     protected IoBuffer duplicate0()
/*     */     {
/* 248 */       return new CachedBuffer(CachedBufferAllocator.this, this, buf().duplicate());
/*     */     }
/*     */     
/*     */ 
/*     */     protected IoBuffer slice0()
/*     */     {
/* 254 */       return new CachedBuffer(CachedBufferAllocator.this, this, buf().slice());
/*     */     }
/*     */     
/*     */ 
/*     */     protected IoBuffer asReadOnlyBuffer0()
/*     */     {
/* 260 */       return new CachedBuffer(CachedBufferAllocator.this, this, buf().asReadOnlyBuffer());
/*     */     }
/*     */     
/*     */ 
/*     */     public byte[] array()
/*     */     {
/* 266 */       return buf().array();
/*     */     }
/*     */     
/*     */ 
/*     */     public int arrayOffset()
/*     */     {
/* 272 */       return buf().arrayOffset();
/*     */     }
/*     */     
/*     */ 
/*     */     public boolean hasArray()
/*     */     {
/* 278 */       return buf().hasArray();
/*     */     }
/*     */     
/*     */ 
/*     */     public void free()
/*     */     {
/* 284 */       free(this.buf);
/* 285 */       this.buf = null;
/*     */     }
/*     */     
/*     */     private void free(ByteBuffer oldBuf)
/*     */     {
/* 290 */       if ((oldBuf == null) || (oldBuf.capacity() > CachedBufferAllocator.this.maxCachedBufferSize) || (oldBuf.isReadOnly()) || (isDerived()) || (Thread.currentThread() != this.ownerThread)) {
/*     */         return;
/*     */       }
/*     */       
/*     */       Queue<CachedBuffer> pool;
/*     */       
/*     */       Queue<CachedBuffer> pool;
/* 297 */       if (oldBuf.isDirect()) {
/* 298 */         pool = (Queue)((Map)CachedBufferAllocator.this.directBuffers.get()).get(Integer.valueOf(oldBuf.capacity()));
/*     */       }
/*     */       else {
/* 301 */         pool = (Queue)((Map)CachedBufferAllocator.this.heapBuffers.get()).get(Integer.valueOf(oldBuf.capacity()));
/*     */       }
/*     */       
/* 304 */       if (pool == null) {
/* 305 */         return;
/*     */       }
/*     */       
/*     */ 
/* 309 */       if ((CachedBufferAllocator.this.maxPoolSize == 0) || (pool.size() < CachedBufferAllocator.this.maxPoolSize)) {
/* 310 */         pool.offer(new CachedBuffer(CachedBufferAllocator.this, oldBuf));
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\buffer\CachedBufferAllocator.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */