/*     */ package com.google.code.yanf4j.nio.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.config.Configuration;
/*     */ import com.google.code.yanf4j.core.EventType;
/*     */ import com.google.code.yanf4j.core.Session;
/*     */ import java.io.IOException;
/*     */ import java.nio.channels.SelectableChannel;
/*     */ import java.util.concurrent.atomic.AtomicInteger;
/*     */ import org.slf4j.Logger;
/*     */ import org.slf4j.LoggerFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SelectorManager
/*     */ {
/*     */   private final Reactor[] reactorSet;
/*  23 */   private final AtomicInteger sets = new AtomicInteger(0);
/*     */   
/*     */   private final NioController controller;
/*     */   
/*     */   private final int dividend;
/*     */   private int reactorReadyCount;
/*     */   private volatile boolean started;
/*     */   
/*     */   public SelectorManager(int selectorPoolSize, NioController controller, Configuration conf)
/*     */     throws IOException
/*     */   {
/*  34 */     if (selectorPoolSize <= 0) {
/*  35 */       throw new IllegalArgumentException("selectorPoolSize<=0");
/*     */     }
/*  37 */     log.info("Creating " + selectorPoolSize + " reactors...");
/*  38 */     this.reactorSet = new Reactor[selectorPoolSize];
/*  39 */     this.controller = controller;
/*  40 */     for (int i = 0; i < selectorPoolSize; i++) {
/*  41 */       this.reactorSet[i] = new Reactor(this, conf, i);
/*     */     }
/*  43 */     this.dividend = (this.reactorSet.length - 1);
/*     */   }
/*     */   
/*     */ 
/*     */   public int getSelectorCount()
/*     */   {
/*  49 */     return this.reactorSet == null ? 0 : this.reactorSet.length;
/*     */   }
/*     */   
/*     */   public synchronized void start() {
/*  53 */     if (this.started) {
/*  54 */       return;
/*     */     }
/*  56 */     this.started = true;
/*  57 */     for (Reactor reactor : this.reactorSet) {
/*  58 */       reactor.start();
/*     */     }
/*     */   }
/*     */   
/*     */   Reactor getReactorFromSession(Session session) {
/*  63 */     Reactor reactor = (Reactor)session.getAttribute(REACTOR_ATTRIBUTE);
/*     */     
/*  65 */     if (reactor == null) {
/*  66 */       reactor = nextReactor();
/*  67 */       Reactor oldReactor = (Reactor)session.setAttributeIfAbsent(REACTOR_ATTRIBUTE, reactor);
/*     */       
/*  69 */       if (oldReactor != null) {
/*  70 */         reactor = oldReactor;
/*     */       }
/*     */     }
/*  73 */     return reactor;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Reactor getReactorByIndex(int index)
/*     */   {
/*  83 */     if ((index < 0) || (index > this.reactorSet.length - 1)) {
/*  84 */       throw new ArrayIndexOutOfBoundsException();
/*     */     }
/*  86 */     return this.reactorSet[index];
/*     */   }
/*     */   
/*     */   public synchronized void stop() {
/*  90 */     if (!this.started) {
/*  91 */       return;
/*     */     }
/*  93 */     this.started = false;
/*  94 */     for (Reactor reactor : this.reactorSet) {
/*  95 */       reactor.interrupt();
/*     */     }
/*     */   }
/*     */   
/*  99 */   public static final String REACTOR_ATTRIBUTE = System.currentTimeMillis() + "_Reactor_Attribute";
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Reactor registerChannel(SelectableChannel channel, int ops, Object attachment)
/*     */   {
/* 112 */     awaitReady();
/* 113 */     int index = 0;
/*     */     
/* 115 */     if ((ops == 16) || (ops == 8)) {
/* 116 */       index = 0;
/*     */     } else {
/* 118 */       index = this.sets.incrementAndGet() % this.dividend + 1;
/*     */     }
/* 120 */     Reactor reactor = this.reactorSet[index];
/* 121 */     reactor.registerChannel(channel, ops, attachment);
/* 122 */     return reactor;
/*     */   }
/*     */   
/*     */   void awaitReady()
/*     */   {
/* 127 */     synchronized (this) {
/* 128 */       while ((!this.started) || (this.reactorReadyCount != this.reactorSet.length)) {
/*     */         try {
/* 130 */           wait(1000L);
/*     */         } catch (InterruptedException e) {
/* 132 */           Thread.currentThread().interrupt();
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Reactor nextReactor()
/*     */   {
/* 144 */     if (this.dividend > 0) {
/* 145 */       return this.reactorSet[(this.sets.incrementAndGet() % this.dividend + 1)];
/*     */     }
/* 147 */     return this.reactorSet[0];
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void registerSession(Session session, EventType event)
/*     */   {
/* 158 */     if ((session.isClosed()) && (event != EventType.UNREGISTER)) {
/* 159 */       return;
/*     */     }
/* 161 */     Reactor reactor = (Reactor)session.getAttribute(REACTOR_ATTRIBUTE);
/*     */     
/* 163 */     if (reactor == null) {
/* 164 */       reactor = nextReactor();
/* 165 */       Reactor oldReactor = (Reactor)session.setAttributeIfAbsent(REACTOR_ATTRIBUTE, reactor);
/*     */       
/* 167 */       if (oldReactor != null) {
/* 168 */         reactor = oldReactor;
/*     */       }
/*     */     }
/* 171 */     reactor.registerSession(session, event);
/*     */   }
/*     */   
/*     */   public NioController getController() {
/* 175 */     return this.controller;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   synchronized void notifyReady()
/*     */   {
/* 182 */     this.reactorReadyCount += 1;
/* 183 */     if (this.reactorReadyCount == this.reactorSet.length) {
/* 184 */       this.controller.notifyReady();
/* 185 */       notifyAll();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/* 190 */   private static final Logger log = LoggerFactory.getLogger(SelectorManager.class);
/*     */   
/*     */   public final boolean isStarted()
/*     */   {
/* 194 */     return this.started;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\nio\impl\SelectorManager.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */