/*     */ package com.google.code.yanf4j.util;
/*     */ 
/*     */ import java.nio.ByteBuffer;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ByteBufferUtils
/*     */ {
/*     */   public static final ByteBuffer increaseBufferCapatity(ByteBuffer byteBuffer)
/*     */   {
/*  56 */     if (byteBuffer == null) {
/*  57 */       throw new IllegalArgumentException("buffer is null");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*  63 */     int capacity = byteBuffer.capacity() + 32768;
/*     */     
/*  65 */     if (capacity < 0) {
/*  66 */       throw new IllegalArgumentException("capacity can't be negative");
/*     */     }
/*  68 */     ByteBuffer result = byteBuffer.isDirect() ? ByteBuffer.allocateDirect(capacity) : ByteBuffer.allocate(capacity);
/*     */     
/*  70 */     result.order(byteBuffer.order());
/*  71 */     byteBuffer.flip();
/*  72 */     result.put(byteBuffer);
/*  73 */     return result;
/*     */   }
/*     */   
/*     */   public static final void flip(ByteBuffer[] buffers) {
/*  77 */     if (buffers == null) {
/*  78 */       return;
/*     */     }
/*  80 */     for (ByteBuffer buffer : buffers) {
/*  81 */       if (buffer != null) {
/*  82 */         buffer.flip();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public static final ByteBuffer gather(ByteBuffer[] buffers) {
/*  88 */     if ((buffers == null) || (buffers.length == 0)) {
/*  89 */       return null;
/*     */     }
/*  91 */     ByteBuffer result = ByteBuffer.allocate(remaining(buffers));
/*  92 */     result.order(buffers[0].order());
/*  93 */     for (int i = 0; i < buffers.length; i++) {
/*  94 */       if (buffers[i] != null) {
/*  95 */         result.put(buffers[i]);
/*     */       }
/*     */     }
/*  98 */     result.flip();
/*  99 */     return result;
/*     */   }
/*     */   
/*     */   public static final int remaining(ByteBuffer[] buffers) {
/* 103 */     if (buffers == null) {
/* 104 */       return 0;
/*     */     }
/* 106 */     int remaining = 0;
/* 107 */     for (int i = 0; i < buffers.length; i++) {
/* 108 */       if (buffers[i] != null) {
/* 109 */         remaining += buffers[i].remaining();
/*     */       }
/*     */     }
/* 112 */     return remaining;
/*     */   }
/*     */   
/*     */   public static final void clear(ByteBuffer[] buffers) {
/* 116 */     if (buffers == null) {
/* 117 */       return;
/*     */     }
/* 119 */     for (ByteBuffer buffer : buffers) {
/* 120 */       if (buffer != null) {
/* 121 */         buffer.clear();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public static final String toHex(byte b) {
/* 127 */     return "" + "0123456789ABCDEF".charAt(0xF & b >> 4) + "0123456789ABCDEF".charAt(b & 0xF);
/*     */   }
/*     */   
/*     */   public static final int indexOf(ByteBuffer buffer, ByteBuffer pattern)
/*     */   {
/* 132 */     if ((pattern == null) || (buffer == null)) {
/* 133 */       return -1;
/*     */     }
/* 135 */     int n = buffer.remaining();
/* 136 */     int m = pattern.remaining();
/* 137 */     int patternPos = pattern.position();
/* 138 */     int bufferPos = buffer.position();
/* 139 */     if (n < m) {
/* 140 */       return -1;
/*     */     }
/* 142 */     for (int s = 0; s <= n - m; s++) {
/* 143 */       boolean match = true;
/* 144 */       for (int i = 0; i < m; i++) {
/* 145 */         if (buffer.get(s + i + bufferPos) != pattern.get(patternPos + i))
/*     */         {
/* 147 */           match = false;
/* 148 */           break;
/*     */         }
/*     */       }
/* 151 */       if (match) {
/* 152 */         return bufferPos + s;
/*     */       }
/*     */     }
/* 155 */     return -1;
/*     */   }
/*     */   
/*     */   public static final int indexOf(ByteBuffer buffer, ByteBuffer pattern, int offset)
/*     */   {
/* 160 */     if (offset < 0) {
/* 161 */       throw new IllegalArgumentException("offset must be greater than 0");
/*     */     }
/* 163 */     if ((pattern == null) || (buffer == null)) {
/* 164 */       return -1;
/*     */     }
/* 166 */     int patternPos = pattern.position();
/* 167 */     int n = buffer.remaining();
/* 168 */     int m = pattern.remaining();
/* 169 */     if (n < m) {
/* 170 */       return -1;
/*     */     }
/* 172 */     if ((offset < buffer.position()) || (offset > buffer.limit())) {
/* 173 */       return -1;
/*     */     }
/* 175 */     for (int s = 0; s <= n - m; s++) {
/* 176 */       boolean match = true;
/* 177 */       for (int i = 0; i < m; i++) {
/* 178 */         if (buffer.get(s + i + offset) != pattern.get(patternPos + i)) {
/* 179 */           match = false;
/* 180 */           break;
/*     */         }
/*     */       }
/* 183 */       if (match) {
/* 184 */         return offset + s;
/*     */       }
/*     */     }
/* 187 */     return -1;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final boolean hasRemaining(ByteBuffer[] buffers)
/*     */   {
/* 198 */     if (buffers == null) {
/* 199 */       return false;
/*     */     }
/* 201 */     for (int i = 0; i < buffers.length; i++) {
/* 202 */       if ((buffers[i] != null) && (buffers[i].hasRemaining())) {
/* 203 */         return true;
/*     */       }
/*     */     }
/* 206 */     return false;
/*     */   }
/*     */   
/*     */   public static final int uByte(byte b) {
/* 210 */     return b & 0xFF;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\util\ByteBufferUtils.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */