/*     */ package com.sun.xml.bind.v2.runtime.property;
/*     */ 
/*     */ import com.sun.xml.bind.api.AccessorException;
/*     */ import com.sun.xml.bind.v2.ClassFactory;
/*     */ import com.sun.xml.bind.v2.model.core.PropertyKind;
/*     */ import com.sun.xml.bind.v2.model.nav.ReflectionNavigator;
/*     */ import com.sun.xml.bind.v2.model.runtime.RuntimeMapPropertyInfo;
/*     */ import com.sun.xml.bind.v2.runtime.JAXBContextImpl;
/*     */ import com.sun.xml.bind.v2.runtime.JaxBeanInfo;
/*     */ import com.sun.xml.bind.v2.runtime.Name;
/*     */ import com.sun.xml.bind.v2.runtime.NameBuilder;
/*     */ import com.sun.xml.bind.v2.runtime.XMLSerializer;
/*     */ import com.sun.xml.bind.v2.runtime.reflect.Accessor;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.ChildLoader;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.Loader;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.Receiver;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.TagName;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.UnmarshallingContext.State;
/*     */ import com.sun.xml.bind.v2.util.QNameMap;
/*     */ import java.io.IOException;
/*     */ import java.util.Arrays;
/*     */ import java.util.Collection;
/*     */ import java.util.Collections;
/*     */ import java.util.HashMap;
/*     */ import java.util.LinkedHashMap;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.TreeMap;
/*     */ import javax.xml.namespace.QName;
/*     */ import javax.xml.stream.XMLStreamException;
/*     */ import org.xml.sax.SAXException;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ final class SingleMapNodeProperty<BeanT, ValueT extends Map>
/*     */   extends PropertyImpl<BeanT>
/*     */ {
/*     */   private final Accessor<BeanT, ValueT> acc;
/*     */   private final Name tagName;
/*     */   private final Name entryTag;
/*     */   private final Name keyTag;
/*     */   private final Name valueTag;
/*     */   private final boolean nillable;
/*     */   private JaxBeanInfo keyBeanInfo;
/*     */   private JaxBeanInfo valueBeanInfo;
/*     */   private final Class<? extends ValueT> mapImplClass;
/*     */   
/*     */   public SingleMapNodeProperty(JAXBContextImpl context, RuntimeMapPropertyInfo prop)
/*     */   {
/*  65 */     super(context, prop);
/*  66 */     this.acc = prop.getAccessor().optimize();
/*  67 */     this.tagName = context.nameBuilder.createElementName(prop.getXmlName());
/*  68 */     this.entryTag = context.nameBuilder.createElementName("", "entry");
/*  69 */     this.keyTag = context.nameBuilder.createElementName("", "key");
/*  70 */     this.valueTag = context.nameBuilder.createElementName("", "value");
/*  71 */     this.nillable = prop.isCollectionNillable();
/*  72 */     this.keyBeanInfo = context.getOrCreate(prop.getKeyType());
/*  73 */     this.valueBeanInfo = context.getOrCreate(prop.getValueType());
/*     */     
/*     */ 
/*  76 */     Class<ValueT> sig = ReflectionNavigator.REFLECTION.erasure(prop.getRawType());
/*  77 */     this.mapImplClass = ClassFactory.inferImplClass(sig, knownImplClasses);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*  82 */   private static final Class[] knownImplClasses = { HashMap.class, TreeMap.class, LinkedHashMap.class };
/*     */   private Loader keyLoader;
/*     */   private Loader valueLoader;
/*     */   
/*     */   public void reset(BeanT bean) throws AccessorException {
/*  87 */     this.acc.set(bean, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public String getIdValue(BeanT bean)
/*     */   {
/*  95 */     return null;
/*     */   }
/*     */   
/*     */   public PropertyKind getKind() {
/*  99 */     return PropertyKind.MAP;
/*     */   }
/*     */   
/*     */   public void buildChildElementUnmarshallers(UnmarshallerChain chain, QNameMap<ChildLoader> handlers) {
/* 103 */     this.keyLoader = this.keyBeanInfo.getLoader(chain.context, true);
/* 104 */     this.valueLoader = this.valueBeanInfo.getLoader(chain.context, true);
/* 105 */     handlers.put(this.tagName, new ChildLoader(this.itemsLoader, null));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 116 */   private final Loader itemsLoader = new Loader(false)
/*     */   {
/*     */     public void startElement(UnmarshallingContext.State state, TagName ea) throws SAXException
/*     */     {
/*     */       try {
/* 121 */         BeanT target = state.prev.target;
/* 122 */         ValueT map = (Map)SingleMapNodeProperty.this.acc.get(target);
/* 123 */         if (map == null) {
/* 124 */           map = (Map)ClassFactory.create(SingleMapNodeProperty.this.mapImplClass);
/* 125 */           SingleMapNodeProperty.this.acc.set(target, map);
/*     */         }
/* 127 */         map.clear();
/* 128 */         state.target = map;
/*     */       }
/*     */       catch (AccessorException e) {
/* 131 */         handleGenericException(e, true);
/* 132 */         state.target = new HashMap();
/*     */       }
/*     */     }
/*     */     
/*     */     public void childElement(UnmarshallingContext.State state, TagName ea) throws SAXException
/*     */     {
/* 138 */       if (ea.matches(SingleMapNodeProperty.this.entryTag)) {
/* 139 */         state.loader = SingleMapNodeProperty.this.entryLoader;
/*     */       } else {
/* 141 */         super.childElement(state, ea);
/*     */       }
/*     */     }
/*     */     
/*     */     public Collection<QName> getExpectedChildElements()
/*     */     {
/* 147 */       return Collections.singleton(SingleMapNodeProperty.this.entryTag.toQName());
/*     */     }
/*     */   };
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 156 */   private final Loader entryLoader = new Loader(false)
/*     */   {
/*     */     public void startElement(UnmarshallingContext.State state, TagName ea) {
/* 159 */       state.target = new Object[2];
/*     */     }
/*     */     
/*     */     public void leaveElement(UnmarshallingContext.State state, TagName ea)
/*     */     {
/* 164 */       Object[] keyValue = (Object[])state.target;
/* 165 */       Map map = (Map)state.prev.target;
/* 166 */       map.put(keyValue[0], keyValue[1]);
/*     */     }
/*     */     
/*     */     public void childElement(UnmarshallingContext.State state, TagName ea) throws SAXException
/*     */     {
/* 171 */       if (ea.matches(SingleMapNodeProperty.this.keyTag)) {
/* 172 */         state.loader = SingleMapNodeProperty.this.keyLoader;
/* 173 */         state.receiver = SingleMapNodeProperty.keyReceiver;
/* 174 */         return;
/*     */       }
/* 176 */       if (ea.matches(SingleMapNodeProperty.this.valueTag)) {
/* 177 */         state.loader = SingleMapNodeProperty.this.valueLoader;
/* 178 */         state.receiver = SingleMapNodeProperty.valueReceiver;
/* 179 */         return;
/*     */       }
/* 181 */       super.childElement(state, ea);
/*     */     }
/*     */     
/*     */     public Collection<QName> getExpectedChildElements()
/*     */     {
/* 186 */       return Arrays.asList(new QName[] { SingleMapNodeProperty.this.keyTag.toQName(), SingleMapNodeProperty.this.valueTag.toQName() });
/*     */     }
/*     */   };
/*     */   
/*     */   private static final class ReceiverImpl implements Receiver {
/*     */     private final int index;
/*     */     
/* 193 */     public ReceiverImpl(int index) { this.index = index; }
/*     */     
/*     */     public void receive(UnmarshallingContext.State state, Object o) {
/* 196 */       ((Object[])state.target)[this.index] = o;
/*     */     }
/*     */   }
/*     */   
/* 200 */   private static final Receiver keyReceiver = new ReceiverImpl(0);
/* 201 */   private static final Receiver valueReceiver = new ReceiverImpl(1);
/*     */   
/*     */ 
/*     */   public void serializeBody(BeanT o, XMLSerializer w, Object outerPeer)
/*     */     throws SAXException, AccessorException, IOException, XMLStreamException
/*     */   {
/* 207 */     ValueT v = (Map)this.acc.get(o);
/* 208 */     if (v != null) {
/* 209 */       bareStartTag(w, this.tagName, v);
/* 210 */       for (Map.Entry e : v.entrySet()) {
/* 211 */         bareStartTag(w, this.entryTag, null);
/*     */         
/* 213 */         Object key = e.getKey();
/* 214 */         if (key != null) {
/* 215 */           w.startElement(this.keyTag, key);
/* 216 */           w.childAsXsiType(key, this.fieldName, this.keyBeanInfo);
/* 217 */           w.endElement();
/*     */         }
/*     */         
/* 220 */         Object value = e.getValue();
/* 221 */         if (value != null) {
/* 222 */           w.startElement(this.valueTag, value);
/* 223 */           w.childAsXsiType(value, this.fieldName, this.valueBeanInfo);
/* 224 */           w.endElement();
/*     */         }
/*     */         
/* 227 */         w.endElement();
/*     */       }
/* 229 */       w.endElement();
/*     */     }
/* 231 */     else if (this.nillable) {
/* 232 */       w.startElement(this.tagName, null);
/* 233 */       w.writeXsiNilTrue();
/* 234 */       w.endElement();
/*     */     }
/*     */   }
/*     */   
/*     */   private void bareStartTag(XMLSerializer w, Name tagName, Object peer) throws IOException, XMLStreamException, SAXException {
/* 239 */     w.startElement(tagName, peer);
/* 240 */     w.endNamespaceDecls(peer);
/* 241 */     w.endAttributes();
/*     */   }
/*     */   
/*     */   public Accessor getElementPropertyAccessor(String nsUri, String localName)
/*     */   {
/* 246 */     if (this.tagName.equals(nsUri, localName))
/* 247 */       return this.acc;
/* 248 */     return null;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\target\myautosupport\WEB-INF\lib\jaxb-impl-2.0.3.jar!\com\sun\xml\bind\v2\runtime\property\SingleMapNodeProperty.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */