/*      */ package org.codehaus.jackson.impl;
/*      */ 
/*      */ import java.io.IOException;
/*      */ import java.io.Reader;
/*      */ import org.codehaus.jackson.Base64Variant;
/*      */ import org.codehaus.jackson.JsonParseException;
/*      */ import org.codehaus.jackson.JsonParser.Feature;
/*      */ import org.codehaus.jackson.JsonToken;
/*      */ import org.codehaus.jackson.ObjectCodec;
/*      */ import org.codehaus.jackson.io.IOContext;
/*      */ import org.codehaus.jackson.sym.CharsToNameCanonicalizer;
/*      */ import org.codehaus.jackson.util.ByteArrayBuilder;
/*      */ import org.codehaus.jackson.util.CharTypes;
/*      */ import org.codehaus.jackson.util.TextBuffer;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public final class ReaderBasedParser
/*      */   extends ReaderBasedNumericParser
/*      */ {
/*      */   protected ObjectCodec _objectCodec;
/*      */   protected final CharsToNameCanonicalizer _symbols;
/*   39 */   protected boolean _tokenIncomplete = false;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ReaderBasedParser(IOContext ioCtxt, int features, Reader r, ObjectCodec codec, CharsToNameCanonicalizer st)
/*      */   {
/*   50 */     super(ioCtxt, features, r);
/*   51 */     this._objectCodec = codec;
/*   52 */     this._symbols = st;
/*      */   }
/*      */   
/*      */   public ObjectCodec getCodec()
/*      */   {
/*   57 */     return this._objectCodec;
/*      */   }
/*      */   
/*      */   public void setCodec(ObjectCodec c)
/*      */   {
/*   62 */     this._objectCodec = c;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final String getText()
/*      */     throws IOException, JsonParseException
/*      */   {
/*   81 */     JsonToken t = this._currToken;
/*   82 */     if (t == JsonToken.VALUE_STRING) {
/*   83 */       if (this._tokenIncomplete) {
/*   84 */         this._tokenIncomplete = false;
/*   85 */         _finishString();
/*      */       }
/*   87 */       return this._textBuffer.contentsAsString();
/*      */     }
/*   89 */     return _getText2(t);
/*      */   }
/*      */   
/*      */   protected final String _getText2(JsonToken t)
/*      */   {
/*   94 */     if (t == null) {
/*   95 */       return null;
/*      */     }
/*   97 */     switch (t) {
/*      */     case FIELD_NAME: 
/*   99 */       return this._parsingContext.getCurrentName();
/*      */     
/*      */ 
/*      */     case VALUE_STRING: 
/*      */     case VALUE_NUMBER_INT: 
/*      */     case VALUE_NUMBER_FLOAT: 
/*  105 */       return this._textBuffer.contentsAsString();
/*      */     }
/*  107 */     return t.asString();
/*      */   }
/*      */   
/*      */ 
/*      */   public char[] getTextCharacters()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  114 */     if (this._currToken != null) {
/*  115 */       switch (this._currToken)
/*      */       {
/*      */       case FIELD_NAME: 
/*  118 */         if (!this._nameCopied) {
/*  119 */           String name = this._parsingContext.getCurrentName();
/*  120 */           int nameLen = name.length();
/*  121 */           if (this._nameCopyBuffer == null) {
/*  122 */             this._nameCopyBuffer = this._ioContext.allocNameCopyBuffer(nameLen);
/*  123 */           } else if (this._nameCopyBuffer.length < nameLen) {
/*  124 */             this._nameCopyBuffer = new char[nameLen];
/*      */           }
/*  126 */           name.getChars(0, nameLen, this._nameCopyBuffer, 0);
/*  127 */           this._nameCopied = true;
/*      */         }
/*  129 */         return this._nameCopyBuffer;
/*      */       
/*      */       case VALUE_STRING: 
/*  132 */         if (this._tokenIncomplete) {
/*  133 */           this._tokenIncomplete = false;
/*  134 */           _finishString();
/*      */         }
/*      */       
/*      */       case VALUE_NUMBER_INT: 
/*      */       case VALUE_NUMBER_FLOAT: 
/*  139 */         return this._textBuffer.getTextBuffer();
/*      */       }
/*      */       
/*  142 */       return this._currToken.asCharArray();
/*      */     }
/*      */     
/*  145 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */   public int getTextLength()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  152 */     if (this._currToken != null) {
/*  153 */       switch (this._currToken)
/*      */       {
/*      */       case FIELD_NAME: 
/*  156 */         return this._parsingContext.getCurrentName().length();
/*      */       case VALUE_STRING: 
/*  158 */         if (this._tokenIncomplete) {
/*  159 */           this._tokenIncomplete = false;
/*  160 */           _finishString();
/*      */         }
/*      */       
/*      */       case VALUE_NUMBER_INT: 
/*      */       case VALUE_NUMBER_FLOAT: 
/*  165 */         return this._textBuffer.size();
/*      */       }
/*      */       
/*  168 */       return this._currToken.asCharArray().length;
/*      */     }
/*      */     
/*  171 */     return 0;
/*      */   }
/*      */   
/*      */ 
/*      */   public int getTextOffset()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  178 */     if (this._currToken != null) {
/*  179 */       switch (this._currToken) {
/*      */       case FIELD_NAME: 
/*  181 */         return 0;
/*      */       case VALUE_STRING: 
/*  183 */         if (this._tokenIncomplete) {
/*  184 */           this._tokenIncomplete = false;
/*  185 */           _finishString();
/*      */         }
/*      */       
/*      */       case VALUE_NUMBER_INT: 
/*      */       case VALUE_NUMBER_FLOAT: 
/*  190 */         return this._textBuffer.getTextOffset();
/*      */       }
/*      */     }
/*  193 */     return 0;
/*      */   }
/*      */   
/*      */ 
/*      */   public byte[] getBinaryValue(Base64Variant b64variant)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  200 */     if ((this._currToken != JsonToken.VALUE_STRING) && ((this._currToken != JsonToken.VALUE_EMBEDDED_OBJECT) || (this._binaryValue == null)))
/*      */     {
/*  202 */       _reportError("Current token (" + this._currToken + ") not VALUE_STRING or VALUE_EMBEDDED_OBJECT, can not access as binary");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  207 */     if (this._tokenIncomplete) {
/*      */       try {
/*  209 */         this._binaryValue = _decodeBase64(b64variant);
/*      */       } catch (IllegalArgumentException iae) {
/*  211 */         throw _constructError("Failed to decode VALUE_STRING as base64 (" + b64variant + "): " + iae.getMessage());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*  216 */       this._tokenIncomplete = false;
/*      */     }
/*  218 */     return this._binaryValue;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonToken nextToken()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  235 */     this._numTypesValid = 0;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  241 */     if (this._currToken == JsonToken.FIELD_NAME) {
/*  242 */       return _nextAfterName();
/*      */     }
/*  244 */     if (this._tokenIncomplete) {
/*  245 */       _skipString();
/*      */     }
/*  247 */     int i = _skipWSOrEnd();
/*  248 */     if (i < 0)
/*      */     {
/*      */ 
/*      */ 
/*  252 */       close();
/*  253 */       return this._currToken = null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  259 */     this._tokenInputTotal = (this._currInputProcessed + this._inputPtr - 1L);
/*  260 */     this._tokenInputRow = this._currInputRow;
/*  261 */     this._tokenInputCol = (this._inputPtr - this._currInputRowStart - 1);
/*      */     
/*      */ 
/*  264 */     this._binaryValue = null;
/*      */     
/*      */ 
/*  267 */     if (i == 93) {
/*  268 */       if (!this._parsingContext.inArray()) {
/*  269 */         _reportMismatchedEndMarker(i, '}');
/*      */       }
/*  271 */       this._parsingContext = this._parsingContext.getParent();
/*  272 */       return this._currToken = JsonToken.END_ARRAY;
/*      */     }
/*  274 */     if (i == 125) {
/*  275 */       if (!this._parsingContext.inObject()) {
/*  276 */         _reportMismatchedEndMarker(i, ']');
/*      */       }
/*  278 */       this._parsingContext = this._parsingContext.getParent();
/*  279 */       return this._currToken = JsonToken.END_OBJECT;
/*      */     }
/*      */     
/*      */ 
/*  283 */     if (this._parsingContext.expectComma()) {
/*  284 */       if (i != 44) {
/*  285 */         _reportUnexpectedChar(i, "was expecting comma to separate " + this._parsingContext.getTypeDesc() + " entries");
/*      */       }
/*  287 */       i = _skipWS();
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  294 */     boolean inObject = this._parsingContext.inObject();
/*  295 */     if (inObject)
/*      */     {
/*  297 */       String name = _parseFieldName(i);
/*  298 */       this._parsingContext.setCurrentName(name);
/*  299 */       this._currToken = JsonToken.FIELD_NAME;
/*  300 */       i = _skipWS();
/*  301 */       if (i != 58) {
/*  302 */         _reportUnexpectedChar(i, "was expecting a colon to separate field name and value");
/*      */       }
/*  304 */       i = _skipWS();
/*      */     }
/*      */     
/*      */ 
/*      */     JsonToken t;
/*      */     
/*      */ 
/*  311 */     switch (i) {
/*      */     case 34: 
/*  313 */       this._tokenIncomplete = true;
/*  314 */       t = JsonToken.VALUE_STRING;
/*  315 */       break;
/*      */     case 91: 
/*  317 */       if (!inObject) {
/*  318 */         this._parsingContext = this._parsingContext.createChildArrayContext(this._tokenInputRow, this._tokenInputCol);
/*      */       }
/*  320 */       t = JsonToken.START_ARRAY;
/*  321 */       break;
/*      */     case 123: 
/*  323 */       if (!inObject) {
/*  324 */         this._parsingContext = this._parsingContext.createChildObjectContext(this._tokenInputRow, this._tokenInputCol);
/*      */       }
/*  326 */       t = JsonToken.START_OBJECT;
/*  327 */       break;
/*      */     
/*      */ 
/*      */     case 93: 
/*      */     case 125: 
/*  332 */       _reportUnexpectedChar(i, "expected a value");
/*      */     case 116: 
/*  334 */       _matchToken(JsonToken.VALUE_TRUE);
/*  335 */       t = JsonToken.VALUE_TRUE;
/*  336 */       break;
/*      */     case 102: 
/*  338 */       _matchToken(JsonToken.VALUE_FALSE);
/*  339 */       t = JsonToken.VALUE_FALSE;
/*  340 */       break;
/*      */     case 110: 
/*  342 */       _matchToken(JsonToken.VALUE_NULL);
/*  343 */       t = JsonToken.VALUE_NULL;
/*  344 */       break;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     case 45: 
/*      */     case 48: 
/*      */     case 49: 
/*      */     case 50: 
/*      */     case 51: 
/*      */     case 52: 
/*      */     case 53: 
/*      */     case 54: 
/*      */     case 55: 
/*      */     case 56: 
/*      */     case 57: 
/*  361 */       t = parseNumberText(i);
/*  362 */       break;
/*      */     default: 
/*  364 */       t = _handleUnexpectedValue(i);
/*      */     }
/*      */     
/*      */     
/*  368 */     if (inObject) {
/*  369 */       this._nextToken = t;
/*  370 */       return this._currToken;
/*      */     }
/*  372 */     this._currToken = t;
/*  373 */     return t;
/*      */   }
/*      */   
/*      */   private final JsonToken _nextAfterName()
/*      */   {
/*  378 */     this._nameCopied = false;
/*  379 */     JsonToken t = this._nextToken;
/*  380 */     this._nextToken = null;
/*      */     
/*  382 */     if (t == JsonToken.START_ARRAY) {
/*  383 */       this._parsingContext = this._parsingContext.createChildArrayContext(this._tokenInputRow, this._tokenInputCol);
/*  384 */     } else if (t == JsonToken.START_OBJECT) {
/*  385 */       this._parsingContext = this._parsingContext.createChildObjectContext(this._tokenInputRow, this._tokenInputCol);
/*      */     }
/*  387 */     return this._currToken = t;
/*      */   }
/*      */   
/*      */   public void close()
/*      */     throws IOException
/*      */   {
/*  393 */     super.close();
/*  394 */     this._symbols.release();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final String _parseFieldName(int i)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  406 */     if (i != 34) {
/*  407 */       return _handleUnusualFieldName(i);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  413 */     int ptr = this._inputPtr;
/*  414 */     int hash = 0;
/*  415 */     int inputLen = this._inputEnd;
/*      */     
/*  417 */     if (ptr < inputLen) {
/*  418 */       int[] codes = CharTypes.getInputCodeLatin1();
/*  419 */       int maxCode = codes.length;
/*      */       do
/*      */       {
/*  422 */         int ch = this._inputBuffer[ptr];
/*  423 */         if ((ch < maxCode) && (codes[ch] != 0)) {
/*  424 */           if (ch != 34) break;
/*  425 */           int start = this._inputPtr;
/*  426 */           this._inputPtr = (ptr + 1);
/*  427 */           return this._symbols.findSymbol(this._inputBuffer, start, ptr - start, hash);
/*      */         }
/*      */         
/*      */ 
/*  431 */         hash = hash * 31 + ch;
/*  432 */         ptr++;
/*  433 */       } while (ptr < inputLen);
/*      */     }
/*      */     
/*  436 */     int start = this._inputPtr;
/*  437 */     this._inputPtr = ptr;
/*  438 */     return _parseFieldName2(start, hash, 34);
/*      */   }
/*      */   
/*      */   private String _parseFieldName2(int startPtr, int hash, int endChar)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  444 */     this._textBuffer.resetWithShared(this._inputBuffer, startPtr, this._inputPtr - startPtr);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  449 */     char[] outBuf = this._textBuffer.getCurrentSegment();
/*  450 */     int outPtr = this._textBuffer.getCurrentSegmentSize();
/*      */     for (;;)
/*      */     {
/*  453 */       if ((this._inputPtr >= this._inputEnd) && 
/*  454 */         (!loadMore())) {
/*  455 */         _reportInvalidEOF(": was expecting closing '" + (char)endChar + "' for name");
/*      */       }
/*      */       
/*  458 */       char c = this._inputBuffer[(this._inputPtr++)];
/*  459 */       int i = c;
/*  460 */       if (i <= 92) {
/*  461 */         if (i == 92)
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*  466 */           c = _decodeEscaped();
/*  467 */         } else if (i <= endChar) {
/*  468 */           if (i == endChar) {
/*      */             break;
/*      */           }
/*  471 */           if (i < 32) {
/*  472 */             _throwUnquotedSpace(i, "name");
/*      */           }
/*      */         }
/*      */       }
/*  476 */       hash = hash * 31 + i;
/*      */       
/*  478 */       outBuf[(outPtr++)] = c;
/*      */       
/*      */ 
/*  481 */       if (outPtr >= outBuf.length) {
/*  482 */         outBuf = this._textBuffer.finishCurrentSegment();
/*  483 */         outPtr = 0;
/*      */       }
/*      */     }
/*  486 */     this._textBuffer.setCurrentLength(outPtr);
/*      */     
/*  488 */     TextBuffer tb = this._textBuffer;
/*  489 */     char[] buf = tb.getTextBuffer();
/*  490 */     int start = tb.getTextOffset();
/*  491 */     int len = tb.size();
/*      */     
/*  493 */     return this._symbols.findSymbol(buf, start, len, hash);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final String _handleUnusualFieldName(int i)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  509 */     if ((i == 39) && (isEnabled(JsonParser.Feature.ALLOW_SINGLE_QUOTES))) {
/*  510 */       return _parseApostropheFieldName();
/*      */     }
/*      */     
/*  513 */     if (!isEnabled(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES)) {
/*  514 */       _reportUnexpectedChar(i, "was expecting double-quote to start field name");
/*      */     }
/*  516 */     int[] codes = CharTypes.getInputCodeLatin1JsNames();
/*  517 */     int maxCode = codes.length;
/*      */     
/*      */     boolean firstOk;
/*      */     
/*      */     boolean firstOk;
/*  522 */     if (i < maxCode) {
/*  523 */       firstOk = (codes[i] == 0) && ((i < 48) || (i > 57));
/*      */     } else {
/*  525 */       firstOk = Character.isJavaIdentifierPart((char)i);
/*      */     }
/*  527 */     if (!firstOk) {
/*  528 */       _reportUnexpectedChar(i, "was expecting either valid name character (for unquoted name) or double-quote (for quoted) to start field name");
/*      */     }
/*  530 */     int ptr = this._inputPtr;
/*  531 */     int hash = 0;
/*  532 */     int inputLen = this._inputEnd;
/*      */     
/*  534 */     if (ptr < inputLen) {
/*      */       do {
/*  536 */         int ch = this._inputBuffer[ptr];
/*  537 */         if (ch < maxCode) {
/*  538 */           if (codes[ch] != 0) {
/*  539 */             int start = this._inputPtr - 1;
/*  540 */             this._inputPtr = ptr;
/*  541 */             return this._symbols.findSymbol(this._inputBuffer, start, ptr - start, hash);
/*      */           }
/*  543 */         } else if (!Character.isJavaIdentifierPart((char)ch)) {
/*  544 */           int start = this._inputPtr - 1;
/*  545 */           this._inputPtr = ptr;
/*  546 */           return this._symbols.findSymbol(this._inputBuffer, start, ptr - start, hash);
/*      */         }
/*  548 */         hash = hash * 31 + ch;
/*  549 */         ptr++;
/*  550 */       } while (ptr < inputLen);
/*      */     }
/*  552 */     int start = this._inputPtr - 1;
/*  553 */     this._inputPtr = ptr;
/*  554 */     return _parseUnusualFieldName2(start, hash, codes);
/*      */   }
/*      */   
/*      */ 
/*      */   protected final String _parseApostropheFieldName()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  561 */     int ptr = this._inputPtr;
/*  562 */     int hash = 0;
/*  563 */     int inputLen = this._inputEnd;
/*      */     
/*  565 */     if (ptr < inputLen) {
/*  566 */       int[] codes = CharTypes.getInputCodeLatin1();
/*  567 */       int maxCode = codes.length;
/*      */       do
/*      */       {
/*  570 */         int ch = this._inputBuffer[ptr];
/*  571 */         if (ch == 39) {
/*  572 */           int start = this._inputPtr;
/*  573 */           this._inputPtr = (ptr + 1);
/*  574 */           return this._symbols.findSymbol(this._inputBuffer, start, ptr - start, hash);
/*      */         }
/*  576 */         if ((ch < maxCode) && (codes[ch] != 0)) {
/*      */           break;
/*      */         }
/*  579 */         hash = hash * 31 + ch;
/*  580 */         ptr++;
/*  581 */       } while (ptr < inputLen);
/*      */     }
/*      */     
/*  584 */     int start = this._inputPtr;
/*  585 */     this._inputPtr = ptr;
/*      */     
/*  587 */     return _parseFieldName2(start, hash, 39);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final JsonToken _handleUnexpectedValue(int i)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  600 */     switch (i)
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     case 39: 
/*  609 */       if (isEnabled(JsonParser.Feature.ALLOW_SINGLE_QUOTES)) {
/*  610 */         return _handleApostropheValue();
/*      */       }
/*      */       break;
/*      */     case 78: 
/*  614 */       if (_matchToken("NaN", 1)) {
/*  615 */         if (isEnabled(JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS)) {
/*  616 */           return resetAsNaN("NaN", NaN.0D);
/*      */         }
/*  618 */         _reportError("Non-standard token 'NaN': enable JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS to allow");
/*      */       }
/*  620 */       _reportUnexpectedChar(this._inputBuffer[(this._inputPtr++)], "expected 'NaN' or a valid value");
/*  621 */       break;
/*      */     case 43: 
/*  623 */       if ((this._inputPtr >= this._inputEnd) && 
/*  624 */         (!loadMore())) {
/*  625 */         _reportInvalidEOFInValue();
/*      */       }
/*      */       
/*  628 */       return _handleInvalidNumberStart(this._inputBuffer[(this._inputPtr++)], false); }
/*      */     
/*  630 */     _reportUnexpectedChar(i, "expected a valid value (number, String, array, object, 'true', 'false' or 'null')");
/*  631 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final JsonToken _handleApostropheValue()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  640 */     char[] outBuf = this._textBuffer.emptyAndGetCurrentSegment();
/*  641 */     int outPtr = this._textBuffer.getCurrentSegmentSize();
/*      */     for (;;)
/*      */     {
/*  644 */       if ((this._inputPtr >= this._inputEnd) && 
/*  645 */         (!loadMore())) {
/*  646 */         _reportInvalidEOF(": was expecting closing quote for a string value");
/*      */       }
/*      */       
/*  649 */       char c = this._inputBuffer[(this._inputPtr++)];
/*  650 */       int i = c;
/*  651 */       if (i <= 92) {
/*  652 */         if (i == 92)
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*  657 */           c = _decodeEscaped();
/*  658 */         } else if (i <= 39) {
/*  659 */           if (i == 39) {
/*      */             break;
/*      */           }
/*  662 */           if (i < 32) {
/*  663 */             _throwUnquotedSpace(i, "string value");
/*      */           }
/*      */         }
/*      */       }
/*      */       
/*  668 */       if (outPtr >= outBuf.length) {
/*  669 */         outBuf = this._textBuffer.finishCurrentSegment();
/*  670 */         outPtr = 0;
/*      */       }
/*      */       
/*  673 */       outBuf[(outPtr++)] = c;
/*      */     }
/*  675 */     this._textBuffer.setCurrentLength(outPtr);
/*  676 */     return JsonToken.VALUE_STRING;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private String _parseUnusualFieldName2(int startPtr, int hash, int[] codes)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  685 */     this._textBuffer.resetWithShared(this._inputBuffer, startPtr, this._inputPtr - startPtr);
/*  686 */     char[] outBuf = this._textBuffer.getCurrentSegment();
/*  687 */     int outPtr = this._textBuffer.getCurrentSegmentSize();
/*  688 */     int maxCode = codes.length;
/*      */     
/*      */ 
/*  691 */     while ((this._inputPtr < this._inputEnd) || 
/*  692 */       (loadMore()))
/*      */     {
/*      */ 
/*      */ 
/*  696 */       char c = this._inputBuffer[this._inputPtr];
/*  697 */       int i = c;
/*  698 */       if (i <= maxCode ? 
/*  699 */         codes[i] != 0 : 
/*      */         
/*      */ 
/*  702 */         !Character.isJavaIdentifierPart(c)) {
/*      */         break;
/*      */       }
/*  705 */       this._inputPtr += 1;
/*  706 */       hash = hash * 31 + i;
/*      */       
/*  708 */       outBuf[(outPtr++)] = c;
/*      */       
/*      */ 
/*  711 */       if (outPtr >= outBuf.length) {
/*  712 */         outBuf = this._textBuffer.finishCurrentSegment();
/*  713 */         outPtr = 0;
/*      */       }
/*      */     }
/*  716 */     this._textBuffer.setCurrentLength(outPtr);
/*      */     
/*  718 */     TextBuffer tb = this._textBuffer;
/*  719 */     char[] buf = tb.getTextBuffer();
/*  720 */     int start = tb.getTextOffset();
/*  721 */     int len = tb.size();
/*      */     
/*  723 */     return this._symbols.findSymbol(buf, start, len, hash);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void _finishString()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  735 */     int ptr = this._inputPtr;
/*  736 */     int inputLen = this._inputEnd;
/*      */     
/*  738 */     if (ptr < inputLen) {
/*  739 */       int[] codes = CharTypes.getInputCodeLatin1();
/*  740 */       int maxCode = codes.length;
/*      */       do
/*      */       {
/*  743 */         int ch = this._inputBuffer[ptr];
/*  744 */         if ((ch < maxCode) && (codes[ch] != 0)) {
/*  745 */           if (ch != 34) break;
/*  746 */           this._textBuffer.resetWithShared(this._inputBuffer, this._inputPtr, ptr - this._inputPtr);
/*  747 */           this._inputPtr = (ptr + 1);
/*      */           
/*  749 */           return;
/*      */         }
/*      */         
/*      */ 
/*  753 */         ptr++;
/*  754 */       } while (ptr < inputLen);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  760 */     this._textBuffer.resetWithCopy(this._inputBuffer, this._inputPtr, ptr - this._inputPtr);
/*  761 */     this._inputPtr = ptr;
/*  762 */     _finishString2();
/*      */   }
/*      */   
/*      */   protected void _finishString2()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  768 */     char[] outBuf = this._textBuffer.getCurrentSegment();
/*  769 */     int outPtr = this._textBuffer.getCurrentSegmentSize();
/*      */     for (;;)
/*      */     {
/*  772 */       if ((this._inputPtr >= this._inputEnd) && 
/*  773 */         (!loadMore())) {
/*  774 */         _reportInvalidEOF(": was expecting closing quote for a string value");
/*      */       }
/*      */       
/*  777 */       char c = this._inputBuffer[(this._inputPtr++)];
/*  778 */       int i = c;
/*  779 */       if (i <= 92) {
/*  780 */         if (i == 92)
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*  785 */           c = _decodeEscaped();
/*  786 */         } else if (i <= 34) {
/*  787 */           if (i == 34) {
/*      */             break;
/*      */           }
/*  790 */           if (i < 32) {
/*  791 */             _throwUnquotedSpace(i, "string value");
/*      */           }
/*      */         }
/*      */       }
/*      */       
/*  796 */       if (outPtr >= outBuf.length) {
/*  797 */         outBuf = this._textBuffer.finishCurrentSegment();
/*  798 */         outPtr = 0;
/*      */       }
/*      */       
/*  801 */       outBuf[(outPtr++)] = c;
/*      */     }
/*  803 */     this._textBuffer.setCurrentLength(outPtr);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void _skipString()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  814 */     this._tokenIncomplete = false;
/*      */     
/*  816 */     int inputPtr = this._inputPtr;
/*  817 */     int inputLen = this._inputEnd;
/*  818 */     char[] inputBuffer = this._inputBuffer;
/*      */     for (;;)
/*      */     {
/*  821 */       if (inputPtr >= inputLen) {
/*  822 */         this._inputPtr = inputPtr;
/*  823 */         if (!loadMore()) {
/*  824 */           _reportInvalidEOF(": was expecting closing quote for a string value");
/*      */         }
/*  826 */         inputPtr = this._inputPtr;
/*  827 */         inputLen = this._inputEnd;
/*      */       }
/*  829 */       char c = inputBuffer[(inputPtr++)];
/*  830 */       int i = c;
/*  831 */       if (i <= 92) {
/*  832 */         if (i == 92)
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*  837 */           this._inputPtr = inputPtr;
/*  838 */           c = _decodeEscaped();
/*  839 */           inputPtr = this._inputPtr;
/*  840 */           inputLen = this._inputEnd;
/*  841 */         } else if (i <= 34) {
/*  842 */           if (i == 34) {
/*  843 */             this._inputPtr = inputPtr;
/*  844 */             break;
/*      */           }
/*  846 */           if (i < 32) {
/*  847 */             this._inputPtr = inputPtr;
/*  848 */             _throwUnquotedSpace(i, "string value");
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void _matchToken(JsonToken token)
/*      */     throws IOException, JsonParseException
/*      */   {
/*  862 */     String matchStr = token.asString();
/*  863 */     int i = 1;
/*      */     
/*  865 */     for (int len = matchStr.length(); i < len; i++) {
/*  866 */       if ((this._inputPtr >= this._inputEnd) && 
/*  867 */         (!loadMore())) {
/*  868 */         _reportInvalidEOF(" in a value");
/*      */       }
/*      */       
/*  871 */       char c = this._inputBuffer[this._inputPtr];
/*  872 */       if (c != matchStr.charAt(i)) {
/*  873 */         _reportInvalidToken(matchStr.substring(0, i), "'null', 'true' or 'false'");
/*      */       }
/*  875 */       this._inputPtr += 1;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final void _skipCR()
/*      */     throws IOException
/*      */   {
/*  896 */     if (((this._inputPtr < this._inputEnd) || (loadMore())) && 
/*  897 */       (this._inputBuffer[this._inputPtr] == '\n')) {
/*  898 */       this._inputPtr += 1;
/*      */     }
/*      */     
/*  901 */     this._currInputRow += 1;
/*  902 */     this._currInputRowStart = this._inputPtr;
/*      */   }
/*      */   
/*      */   protected final void _skipLF() throws IOException
/*      */   {
/*  907 */     this._currInputRow += 1;
/*  908 */     this._currInputRowStart = this._inputPtr;
/*      */   }
/*      */   
/*      */   private final int _skipWS()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  914 */     while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/*  915 */       int i = this._inputBuffer[(this._inputPtr++)];
/*  916 */       if (i > 32) {
/*  917 */         if (i != 47) {
/*  918 */           return i;
/*      */         }
/*  920 */         _skipComment();
/*  921 */       } else if (i != 32) {
/*  922 */         if (i == 10) {
/*  923 */           _skipLF();
/*  924 */         } else if (i == 13) {
/*  925 */           _skipCR();
/*  926 */         } else if (i != 9) {
/*  927 */           _throwInvalidSpace(i);
/*      */         }
/*      */       }
/*      */     }
/*  931 */     throw _constructError("Unexpected end-of-input within/between " + this._parsingContext.getTypeDesc() + " entries");
/*      */   }
/*      */   
/*      */   private final int _skipWSOrEnd()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  937 */     while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/*  938 */       int i = this._inputBuffer[(this._inputPtr++)];
/*  939 */       if (i > 32) {
/*  940 */         if (i != 47) {
/*  941 */           return i;
/*      */         }
/*  943 */         _skipComment();
/*  944 */       } else if (i != 32) {
/*  945 */         if (i == 10) {
/*  946 */           _skipLF();
/*  947 */         } else if (i == 13) {
/*  948 */           _skipCR();
/*  949 */         } else if (i != 9) {
/*  950 */           _throwInvalidSpace(i);
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*  955 */     _handleEOF();
/*  956 */     return -1;
/*      */   }
/*      */   
/*      */   private final void _skipComment()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  962 */     if (!isEnabled(JsonParser.Feature.ALLOW_COMMENTS)) {
/*  963 */       _reportUnexpectedChar(47, "maybe a (non-standard) comment? (not recognized as one since Feature 'ALLOW_COMMENTS' not enabled for parser)");
/*      */     }
/*      */     
/*  966 */     if ((this._inputPtr >= this._inputEnd) && (!loadMore())) {
/*  967 */       _reportInvalidEOF(" in a comment");
/*      */     }
/*  969 */     char c = this._inputBuffer[(this._inputPtr++)];
/*  970 */     if (c == '/') {
/*  971 */       _skipCppComment();
/*  972 */     } else if (c == '*') {
/*  973 */       _skipCComment();
/*      */     } else {
/*  975 */       _reportUnexpectedChar(c, "was expecting either '*' or '/' for a comment");
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private final void _skipCComment()
/*      */     throws IOException, JsonParseException
/*      */   {
/*  984 */     while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/*  985 */       int i = this._inputBuffer[(this._inputPtr++)];
/*  986 */       if (i <= 42) {
/*  987 */         if (i == 42) {
/*  988 */           if ((this._inputPtr >= this._inputEnd) && (!loadMore())) {
/*      */             break;
/*      */           }
/*  991 */           if (this._inputBuffer[this._inputPtr] == '/') {
/*  992 */             this._inputPtr += 1;
/*      */           }
/*      */           
/*      */ 
/*      */         }
/*  997 */         else if (i < 32) {
/*  998 */           if (i == 10) {
/*  999 */             _skipLF();
/* 1000 */           } else if (i == 13) {
/* 1001 */             _skipCR();
/* 1002 */           } else if (i != 9) {
/* 1003 */             _throwInvalidSpace(i);
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/* 1008 */     _reportInvalidEOF(" in a comment");
/*      */   }
/*      */   
/*      */ 
/*      */   private final void _skipCppComment()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1015 */     while ((this._inputPtr < this._inputEnd) || (loadMore())) {
/* 1016 */       int i = this._inputBuffer[(this._inputPtr++)];
/* 1017 */       if (i < 32) {
/* 1018 */         if (i == 10) {
/* 1019 */           _skipLF();
/* 1020 */           break; }
/* 1021 */         if (i == 13) {
/* 1022 */           _skipCR();
/* 1023 */           break; }
/* 1024 */         if (i != 9) {
/* 1025 */           _throwInvalidSpace(i);
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   protected final char _decodeEscaped()
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1034 */     if ((this._inputPtr >= this._inputEnd) && 
/* 1035 */       (!loadMore())) {
/* 1036 */       _reportInvalidEOF(" in character escape sequence");
/*      */     }
/*      */     
/* 1039 */     char c = this._inputBuffer[(this._inputPtr++)];
/*      */     
/* 1041 */     switch (c)
/*      */     {
/*      */     case 'b': 
/* 1044 */       return '\b';
/*      */     case 't': 
/* 1046 */       return '\t';
/*      */     case 'n': 
/* 1048 */       return '\n';
/*      */     case 'f': 
/* 1050 */       return '\f';
/*      */     case 'r': 
/* 1052 */       return '\r';
/*      */     
/*      */ 
/*      */     case '"': 
/*      */     case '/': 
/*      */     case '\\': 
/* 1058 */       return c;
/*      */     
/*      */     case 'u': 
/*      */       break;
/*      */     
/*      */     default: 
/* 1064 */       return _handleUnrecognizedCharacterEscape(c);
/*      */     }
/*      */     
/*      */     
/* 1068 */     int value = 0;
/* 1069 */     for (int i = 0; i < 4; i++) {
/* 1070 */       if ((this._inputPtr >= this._inputEnd) && 
/* 1071 */         (!loadMore())) {
/* 1072 */         _reportInvalidEOF(" in character escape sequence");
/*      */       }
/*      */       
/* 1075 */       int ch = this._inputBuffer[(this._inputPtr++)];
/* 1076 */       int digit = CharTypes.charToHex(ch);
/* 1077 */       if (digit < 0) {
/* 1078 */         _reportUnexpectedChar(ch, "expected a hex-digit for character escape sequence");
/*      */       }
/* 1080 */       value = value << 4 | digit;
/*      */     }
/* 1082 */     return (char)value;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected byte[] _decodeBase64(Base64Variant b64variant)
/*      */     throws IOException, JsonParseException
/*      */   {
/* 1095 */     ByteArrayBuilder builder = _getByteArrayBuilder();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     for (;;)
/*      */     {
/* 1102 */       if (this._inputPtr >= this._inputEnd) {
/* 1103 */         loadMoreGuaranteed();
/*      */       }
/* 1105 */       char ch = this._inputBuffer[(this._inputPtr++)];
/* 1106 */       if (ch > ' ') {
/* 1107 */         int bits = b64variant.decodeBase64Char(ch);
/* 1108 */         if (bits < 0) {
/* 1109 */           if (ch == '"') {
/* 1110 */             return builder.toByteArray();
/*      */           }
/* 1112 */           bits = _decodeBase64Escape(b64variant, ch, 0);
/* 1113 */           if (bits < 0) {}
/*      */         }
/*      */         else
/*      */         {
/* 1117 */           int decodedData = bits;
/*      */           
/*      */ 
/*      */ 
/* 1121 */           if (this._inputPtr >= this._inputEnd) {
/* 1122 */             loadMoreGuaranteed();
/*      */           }
/* 1124 */           ch = this._inputBuffer[(this._inputPtr++)];
/* 1125 */           bits = b64variant.decodeBase64Char(ch);
/* 1126 */           if (bits < 0) {
/* 1127 */             bits = _decodeBase64Escape(b64variant, ch, 1);
/*      */           }
/* 1129 */           decodedData = decodedData << 6 | bits;
/*      */           
/*      */ 
/* 1132 */           if (this._inputPtr >= this._inputEnd) {
/* 1133 */             loadMoreGuaranteed();
/*      */           }
/* 1135 */           ch = this._inputBuffer[(this._inputPtr++)];
/* 1136 */           bits = b64variant.decodeBase64Char(ch);
/*      */           
/*      */ 
/* 1139 */           if (bits < 0) {
/* 1140 */             if (bits != -2)
/*      */             {
/* 1142 */               if ((ch == '"') && (!b64variant.usesPadding())) {
/* 1143 */                 decodedData >>= 4;
/* 1144 */                 builder.append(decodedData);
/* 1145 */                 return builder.toByteArray();
/*      */               }
/* 1147 */               bits = _decodeBase64Escape(b64variant, ch, 2);
/*      */             }
/* 1149 */             if (bits == -2)
/*      */             {
/* 1151 */               if (this._inputPtr >= this._inputEnd) {
/* 1152 */                 loadMoreGuaranteed();
/*      */               }
/* 1154 */               ch = this._inputBuffer[(this._inputPtr++)];
/* 1155 */               if (!b64variant.usesPaddingChar(ch)) {
/* 1156 */                 throw reportInvalidChar(b64variant, ch, 3, "expected padding character '" + b64variant.getPaddingChar() + "'");
/*      */               }
/*      */               
/* 1159 */               decodedData >>= 4;
/* 1160 */               builder.append(decodedData);
/* 1161 */               continue;
/*      */             }
/*      */           }
/*      */           
/*      */ 
/* 1166 */           decodedData = decodedData << 6 | bits;
/*      */           
/* 1168 */           if (this._inputPtr >= this._inputEnd) {
/* 1169 */             loadMoreGuaranteed();
/*      */           }
/* 1171 */           ch = this._inputBuffer[(this._inputPtr++)];
/* 1172 */           bits = b64variant.decodeBase64Char(ch);
/* 1173 */           if (bits < 0) {
/* 1174 */             if (bits != -2)
/*      */             {
/* 1176 */               if ((ch == '"') && (!b64variant.usesPadding())) {
/* 1177 */                 decodedData >>= 2;
/* 1178 */                 builder.appendTwoBytes(decodedData);
/* 1179 */                 return builder.toByteArray();
/*      */               }
/* 1181 */               bits = _decodeBase64Escape(b64variant, ch, 3);
/*      */             }
/* 1183 */             if (bits == -2)
/*      */             {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1190 */               decodedData >>= 2;
/* 1191 */               builder.appendTwoBytes(decodedData);
/* 1192 */               continue;
/*      */             }
/*      */           }
/*      */           
/*      */ 
/* 1197 */           decodedData = decodedData << 6 | bits;
/* 1198 */           builder.appendThreeBytes(decodedData);
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private final int _decodeBase64Escape(Base64Variant b64variant, char ch, int index) throws IOException, JsonParseException
/*      */   {
/* 1206 */     if (ch != '\\') {
/* 1207 */       throw reportInvalidChar(b64variant, ch, index);
/*      */     }
/* 1209 */     char unescaped = _decodeEscaped();
/*      */     
/* 1211 */     if ((unescaped <= ' ') && 
/* 1212 */       (index == 0)) {
/* 1213 */       return -1;
/*      */     }
/*      */     
/*      */ 
/* 1217 */     int bits = b64variant.decodeBase64Char(unescaped);
/* 1218 */     if (bits < 0) {
/* 1219 */       throw reportInvalidChar(b64variant, unescaped, index);
/*      */     }
/* 1221 */     return bits;
/*      */   }
/*      */   
/*      */   protected IllegalArgumentException reportInvalidChar(Base64Variant b64variant, char ch, int bindex)
/*      */     throws IllegalArgumentException
/*      */   {
/* 1227 */     return reportInvalidChar(b64variant, ch, bindex, null);
/*      */   }
/*      */   
/*      */ 
/*      */   protected IllegalArgumentException reportInvalidChar(Base64Variant b64variant, char ch, int bindex, String msg)
/*      */     throws IllegalArgumentException
/*      */   {
/*      */     String base;
/*      */     
/*      */     String base;
/*      */     
/* 1238 */     if (ch <= ' ') {
/* 1239 */       base = "Illegal white space character (code 0x" + Integer.toHexString(ch) + ") as character #" + (bindex + 1) + " of 4-char base64 unit: can only used between units"; } else { String base;
/* 1240 */       if (b64variant.usesPaddingChar(ch)) {
/* 1241 */         base = "Unexpected padding character ('" + b64variant.getPaddingChar() + "') as character #" + (bindex + 1) + " of 4-char base64 unit: padding only legal as 3rd or 4th character"; } else { String base;
/* 1242 */         if ((!Character.isDefined(ch)) || (Character.isISOControl(ch)))
/*      */         {
/* 1244 */           base = "Illegal character (code 0x" + Integer.toHexString(ch) + ") in base64 content";
/*      */         } else
/* 1246 */           base = "Illegal character '" + ch + "' (code 0x" + Integer.toHexString(ch) + ") in base64 content";
/*      */       } }
/* 1248 */     if (msg != null) {
/* 1249 */       base = base + ": " + msg;
/*      */     }
/* 1251 */     return new IllegalArgumentException(base);
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\impl\ReaderBasedParser.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */