/*     */ package org.codehaus.jackson.map;
/*     */ 
/*     */ import java.io.EOFException;
/*     */ import java.io.File;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.Reader;
/*     */ import java.lang.reflect.Type;
/*     */ import java.net.URL;
/*     */ import java.util.concurrent.ConcurrentHashMap;
/*     */ import org.codehaus.jackson.FormatSchema;
/*     */ import org.codehaus.jackson.JsonFactory;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.JsonParseException;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.ObjectCodec;
/*     */ import org.codehaus.jackson.Version;
/*     */ import org.codehaus.jackson.Versioned;
/*     */ import org.codehaus.jackson.map.deser.StdDeserializationContext;
/*     */ import org.codehaus.jackson.map.type.SimpleType;
/*     */ import org.codehaus.jackson.map.type.TypeFactory;
/*     */ import org.codehaus.jackson.node.JsonNodeFactory;
/*     */ import org.codehaus.jackson.node.NullNode;
/*     */ import org.codehaus.jackson.node.TreeTraversingParser;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ import org.codehaus.jackson.type.TypeReference;
/*     */ import org.codehaus.jackson.util.VersionUtil;
/*     */ 
/*     */ public class ObjectReader extends ObjectCodec implements Versioned
/*     */ {
/*  34 */   private static final JavaType JSON_NODE_TYPE = SimpleType.constructUnsafe(JsonNode.class);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final DeserializationConfig _config;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final ConcurrentHashMap<JavaType, JsonDeserializer<Object>> _rootDeserializers;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final DeserializerProvider _provider;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final JsonFactory _jsonFactory;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final JavaType _valueType;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final Object _valueToUpdate;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final FormatSchema _schema;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected ObjectReader(ObjectMapper mapper, DeserializationConfig config)
/*     */   {
/* 107 */     this(mapper, config, null, null, null);
/*     */   }
/*     */   
/*     */ 
/*     */   protected ObjectReader(ObjectMapper mapper, DeserializationConfig config, JavaType valueType, Object valueToUpdate, FormatSchema schema)
/*     */   {
/* 113 */     this._config = config;
/* 114 */     this._rootDeserializers = mapper._rootDeserializers;
/* 115 */     this._provider = mapper._deserializerProvider;
/* 116 */     this._jsonFactory = mapper._jsonFactory;
/* 117 */     this._valueType = valueType;
/* 118 */     this._valueToUpdate = valueToUpdate;
/* 119 */     if ((valueToUpdate != null) && (valueType.isArrayType())) {
/* 120 */       throw new IllegalArgumentException("Can not update an array value");
/*     */     }
/* 122 */     this._schema = schema;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected ObjectReader(ObjectReader base, DeserializationConfig config, JavaType valueType, Object valueToUpdate, FormatSchema schema)
/*     */   {
/* 131 */     this._config = config;
/*     */     
/* 133 */     this._rootDeserializers = base._rootDeserializers;
/* 134 */     this._provider = base._provider;
/* 135 */     this._jsonFactory = base._jsonFactory;
/*     */     
/* 137 */     this._valueType = valueType;
/* 138 */     this._valueToUpdate = valueToUpdate;
/* 139 */     if ((valueToUpdate != null) && (valueType.isArrayType())) {
/* 140 */       throw new IllegalArgumentException("Can not update an array value");
/*     */     }
/* 142 */     this._schema = schema;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Version version()
/*     */   {
/* 152 */     return VersionUtil.versionFor(getClass());
/*     */   }
/*     */   
/*     */   public ObjectReader withType(JavaType valueType)
/*     */   {
/* 157 */     if (valueType == this._valueType) { return this;
/*     */     }
/* 159 */     return new ObjectReader(this, this._config, valueType, this._valueToUpdate, this._schema);
/*     */   }
/*     */   
/*     */   public ObjectReader withType(Class<?> valueType)
/*     */   {
/* 164 */     return withType(this._config.constructType(valueType));
/*     */   }
/*     */   
/*     */   public ObjectReader withType(Type valueType)
/*     */   {
/* 169 */     return withType(this._config.getTypeFactory().constructType(valueType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectReader withType(TypeReference<?> valueTypeRef)
/*     */   {
/* 177 */     return withType(this._config.getTypeFactory().constructType(valueTypeRef.getType()));
/*     */   }
/*     */   
/*     */ 
/*     */   public ObjectReader withNodeFactory(JsonNodeFactory f)
/*     */   {
/* 183 */     if (f == this._config.getNodeFactory()) return this;
/* 184 */     return new ObjectReader(this, this._config.withNodeFactory(f), this._valueType, this._valueToUpdate, this._schema);
/*     */   }
/*     */   
/*     */   public ObjectReader withValueToUpdate(Object value)
/*     */   {
/* 189 */     if (value == this._valueToUpdate) return this;
/* 190 */     if (value == null) {
/* 191 */       throw new IllegalArgumentException("cat not update null value");
/*     */     }
/* 193 */     JavaType t = this._config.constructType(value.getClass());
/* 194 */     return new ObjectReader(this, this._config, t, value, this._schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public ObjectReader withSchema(FormatSchema schema)
/*     */   {
/* 202 */     if (this._schema == schema) {
/* 203 */       return this;
/*     */     }
/* 205 */     return new ObjectReader(this, this._config, this._valueType, this._valueToUpdate, schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> T readValue(JsonParser jp)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 219 */     return (T)_bind(jp);
/*     */   }
/*     */   
/*     */ 
/*     */   public JsonNode readTree(JsonParser jp)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 226 */     return _bindAsTree(jp);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> T readValue(InputStream src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 239 */     return (T)_bindAndClose(this._jsonFactory.createJsonParser(src));
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T readValue(Reader src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 246 */     return (T)_bindAndClose(this._jsonFactory.createJsonParser(src));
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T readValue(String src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 253 */     return (T)_bindAndClose(this._jsonFactory.createJsonParser(src));
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T readValue(byte[] src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 260 */     return (T)_bindAndClose(this._jsonFactory.createJsonParser(src));
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T readValue(byte[] src, int offset, int length)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 267 */     return (T)_bindAndClose(this._jsonFactory.createJsonParser(src, offset, length));
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T readValue(File src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 274 */     return (T)_bindAndClose(this._jsonFactory.createJsonParser(src));
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T readValue(URL src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 281 */     return (T)_bindAndClose(this._jsonFactory.createJsonParser(src));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> T readValue(JsonNode src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 295 */     return (T)_bindAndClose(treeAsTokens(src));
/*     */   }
/*     */   
/*     */   public JsonNode readTree(InputStream in)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 301 */     return _bindAndCloseAsTree(this._jsonFactory.createJsonParser(in));
/*     */   }
/*     */   
/*     */   public JsonNode readTree(Reader r)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 307 */     return _bindAndCloseAsTree(this._jsonFactory.createJsonParser(r));
/*     */   }
/*     */   
/*     */   public JsonNode readTree(String content)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 313 */     return _bindAndCloseAsTree(this._jsonFactory.createJsonParser(content));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> MappingIterator<T> readValues(JsonParser jp)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 330 */     DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/* 331 */     return new MappingIterator(this._valueType, jp, ctxt, _findRootDeserializer(this._config, this._valueType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> MappingIterator<T> readValues(InputStream src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 342 */     JsonParser jp = this._jsonFactory.createJsonParser(src);
/* 343 */     if (this._schema != null) {
/* 344 */       jp.setSchema(this._schema);
/*     */     }
/* 346 */     DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/* 347 */     return new MappingIterator(this._valueType, jp, ctxt, _findRootDeserializer(this._config, this._valueType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> MappingIterator<T> readValues(Reader src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 358 */     JsonParser jp = this._jsonFactory.createJsonParser(src);
/* 359 */     if (this._schema != null) {
/* 360 */       jp.setSchema(this._schema);
/*     */     }
/* 362 */     DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/* 363 */     return new MappingIterator(this._valueType, jp, ctxt, _findRootDeserializer(this._config, this._valueType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> MappingIterator<T> readValues(String json)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 374 */     JsonParser jp = this._jsonFactory.createJsonParser(json);
/* 375 */     if (this._schema != null) {
/* 376 */       jp.setSchema(this._schema);
/*     */     }
/* 378 */     DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/* 379 */     return new MappingIterator(this._valueType, jp, ctxt, _findRootDeserializer(this._config, this._valueType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> MappingIterator<T> readValues(byte[] src, int offset, int length)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 390 */     JsonParser jp = this._jsonFactory.createJsonParser(src, offset, length);
/* 391 */     if (this._schema != null) {
/* 392 */       jp.setSchema(this._schema);
/*     */     }
/* 394 */     DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/* 395 */     return new MappingIterator(this._valueType, jp, ctxt, _findRootDeserializer(this._config, this._valueType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> MappingIterator<T> readValues(File src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 406 */     JsonParser jp = this._jsonFactory.createJsonParser(src);
/* 407 */     if (this._schema != null) {
/* 408 */       jp.setSchema(this._schema);
/*     */     }
/* 410 */     DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/* 411 */     return new MappingIterator(this._valueType, jp, ctxt, _findRootDeserializer(this._config, this._valueType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <T> MappingIterator<T> readValues(URL src)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 422 */     JsonParser jp = this._jsonFactory.createJsonParser(src);
/* 423 */     if (this._schema != null) {
/* 424 */       jp.setSchema(this._schema);
/*     */     }
/* 426 */     DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/* 427 */     return new MappingIterator(this._valueType, jp, ctxt, _findRootDeserializer(this._config, this._valueType));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object _bind(JsonParser jp)
/*     */     throws IOException, JsonParseException, JsonMappingException
/*     */   {
/* 446 */     JsonToken t = _initForReading(jp);
/* 447 */     Object result; Object result; if ((t == JsonToken.VALUE_NULL) || (t == JsonToken.END_ARRAY) || (t == JsonToken.END_OBJECT)) {
/* 448 */       result = this._valueToUpdate;
/*     */     } else {
/* 450 */       DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/* 451 */       Object result; if (this._valueToUpdate == null) {
/* 452 */         result = _findRootDeserializer(this._config, this._valueType).deserialize(jp, ctxt);
/*     */       } else {
/* 454 */         _findRootDeserializer(this._config, this._valueType).deserialize(jp, ctxt, this._valueToUpdate);
/* 455 */         result = this._valueToUpdate;
/*     */       }
/*     */     }
/*     */     
/* 459 */     jp.clearCurrentToken();
/* 460 */     return result;
/*     */   }
/*     */   
/*     */   protected Object _bindAndClose(JsonParser jp)
/*     */     throws IOException, JsonParseException, JsonMappingException
/*     */   {
/* 466 */     if (this._schema != null) {
/* 467 */       jp.setSchema(this._schema);
/*     */     }
/*     */     try
/*     */     {
/* 471 */       JsonToken t = _initForReading(jp);
/* 472 */       Object result; DeserializationContext ctxt; Object result; if ((t == JsonToken.VALUE_NULL) || (t == JsonToken.END_ARRAY) || (t == JsonToken.END_OBJECT)) {
/* 473 */         result = this._valueToUpdate;
/*     */       } else {
/* 475 */         ctxt = _createDeserializationContext(jp, this._config);
/* 476 */         Object result; if (this._valueToUpdate == null) {
/* 477 */           result = _findRootDeserializer(this._config, this._valueType).deserialize(jp, ctxt);
/*     */         } else {
/* 479 */           _findRootDeserializer(this._config, this._valueType).deserialize(jp, ctxt, this._valueToUpdate);
/* 480 */           result = this._valueToUpdate;
/*     */         }
/*     */       }
/* 483 */       return (DeserializationContext)result;
/*     */     } finally {
/*     */       try {
/* 486 */         jp.close();
/*     */       }
/*     */       catch (IOException ioe) {}
/*     */     }
/*     */   }
/*     */   
/*     */   protected JsonNode _bindAsTree(JsonParser jp)
/*     */     throws IOException, JsonParseException, JsonMappingException
/*     */   {
/* 495 */     JsonToken t = _initForReading(jp);
/* 496 */     JsonNode result; JsonNode result; if ((t == JsonToken.VALUE_NULL) || (t == JsonToken.END_ARRAY) || (t == JsonToken.END_OBJECT)) {
/* 497 */       result = NullNode.instance;
/*     */     } else {
/* 499 */       DeserializationContext ctxt = _createDeserializationContext(jp, this._config);
/*     */       
/* 501 */       result = (JsonNode)_findRootDeserializer(this._config, JSON_NODE_TYPE).deserialize(jp, ctxt);
/*     */     }
/*     */     
/* 504 */     jp.clearCurrentToken();
/* 505 */     return result;
/*     */   }
/*     */   
/*     */   protected JsonNode _bindAndCloseAsTree(JsonParser jp)
/*     */     throws IOException, JsonParseException, JsonMappingException
/*     */   {
/* 511 */     if (this._schema != null) {
/* 512 */       jp.setSchema(this._schema);
/*     */     }
/*     */     try {
/* 515 */       return _bindAsTree(jp);
/*     */     } finally {
/*     */       try {
/* 518 */         jp.close();
/*     */       }
/*     */       catch (IOException ioe) {}
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected static JsonToken _initForReading(JsonParser jp)
/*     */     throws IOException, JsonParseException, JsonMappingException
/*     */   {
/* 530 */     JsonToken t = jp.getCurrentToken();
/* 531 */     if (t == null) {
/* 532 */       t = jp.nextToken();
/* 533 */       if (t == null) {
/* 534 */         throw new EOFException("No content to map to Object due to end of input");
/*     */       }
/*     */     }
/* 537 */     return t;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> _findRootDeserializer(DeserializationConfig cfg, JavaType valueType)
/*     */     throws JsonMappingException
/*     */   {
/* 547 */     JsonDeserializer<Object> deser = (JsonDeserializer)this._rootDeserializers.get(valueType);
/* 548 */     if (deser != null) {
/* 549 */       return deser;
/*     */     }
/*     */     
/*     */ 
/* 553 */     deser = this._provider.findTypedValueDeserializer(cfg, valueType, null);
/* 554 */     if (deser == null) {
/* 555 */       throw new JsonMappingException("Can not find a deserializer for type " + valueType);
/*     */     }
/* 557 */     this._rootDeserializers.put(valueType, deser);
/* 558 */     return deser;
/*     */   }
/*     */   
/*     */   protected DeserializationContext _createDeserializationContext(JsonParser jp, DeserializationConfig cfg)
/*     */   {
/* 563 */     return new StdDeserializationContext(cfg, jp, this._provider);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonNode createArrayNode()
/*     */   {
/* 574 */     return this._config.getNodeFactory().arrayNode();
/*     */   }
/*     */   
/*     */   public JsonNode createObjectNode()
/*     */   {
/* 579 */     return this._config.getNodeFactory().objectNode();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public <T> T readValue(JsonParser jp, Class<T> valueType)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 587 */     return (T)withType(valueType).readValue(jp);
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T readValue(JsonParser jp, TypeReference<?> valueTypeRef)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 594 */     return (T)withType(valueTypeRef).readValue(jp);
/*     */   }
/*     */   
/*     */   public <T> T readValue(JsonParser jp, JavaType valueType)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 600 */     return (T)withType(valueType).readValue(jp);
/*     */   }
/*     */   
/*     */   public JsonParser treeAsTokens(JsonNode n)
/*     */   {
/* 605 */     return new TreeTraversingParser(n, this);
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T treeToValue(JsonNode n, Class<T> valueType)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 612 */     return (T)readValue(treeAsTokens(n), valueType);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void writeTree(JsonGenerator jgen, JsonNode rootNode)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 621 */     throw new UnsupportedOperationException("Not implemented for ObjectReader");
/*     */   }
/*     */   
/*     */   public void writeValue(JsonGenerator jgen, Object value)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 627 */     throw new UnsupportedOperationException("Not implemented for ObjectReader");
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ObjectReader.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */