/*     */ package org.codehaus.jackson.map.introspect;
/*     */ 
/*     */ import java.lang.reflect.Method;
/*     */ import java.lang.reflect.Modifier;
/*     */ import java.util.Collection;
/*     */ import java.util.Map;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*     */ import org.codehaus.jackson.map.ClassIntrospector;
/*     */ import org.codehaus.jackson.map.ClassIntrospector.MixInResolver;
/*     */ import org.codehaus.jackson.map.DeserializationConfig;
/*     */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.MapperConfig;
/*     */ import org.codehaus.jackson.map.SerializationConfig;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BasicClassIntrospector
/*     */   extends ClassIntrospector<BasicBeanDescription>
/*     */ {
/*     */   public static class GetterMethodFilter
/*     */     implements MethodFilter
/*     */   {
/*     */     public boolean includeMethod(Method m)
/*     */     {
/*  30 */       return ClassUtil.hasGetterSignature(m);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static class SetterMethodFilter
/*     */     implements MethodFilter
/*     */   {
/*     */     public boolean includeMethod(Method m)
/*     */     {
/*  49 */       if (Modifier.isStatic(m.getModifiers())) {
/*  50 */         return false;
/*     */       }
/*  52 */       int pcount = m.getParameterTypes().length;
/*     */       
/*  54 */       switch (pcount)
/*     */       {
/*     */       case 1: 
/*  57 */         return true;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       case 2: 
/*  67 */         return true;
/*     */       }
/*  69 */       return false;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final class SetterAndGetterMethodFilter
/*     */     extends BasicClassIntrospector.SetterMethodFilter
/*     */   {
/*     */     public boolean includeMethod(Method m)
/*     */     {
/*  83 */       if (super.includeMethod(m)) {
/*  84 */         return true;
/*     */       }
/*  86 */       if (!ClassUtil.hasGetterSignature(m)) {
/*  87 */         return false;
/*     */       }
/*     */       
/*  90 */       Class<?> rt = m.getReturnType();
/*  91 */       if ((Collection.class.isAssignableFrom(rt)) || (Map.class.isAssignableFrom(rt)))
/*     */       {
/*  93 */         return true;
/*     */       }
/*  95 */       return false;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/* 102 */   public static final GetterMethodFilter DEFAULT_GETTER_FILTER = new GetterMethodFilter(null);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/* 107 */   public static final SetterMethodFilter DEFAULT_SETTER_FILTER = new SetterMethodFilter();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/* 112 */   public static final SetterAndGetterMethodFilter DEFAULT_SETTER_AND_GETTER_FILTER = new SetterAndGetterMethodFilter();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 120 */   public static final BasicClassIntrospector instance = new BasicClassIntrospector();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public BasicBeanDescription forSerialization(SerializationConfig cfg, JavaType type, ClassIntrospector.MixInResolver r)
/*     */   {
/* 134 */     boolean useAnnotations = cfg.isAnnotationProcessingEnabled();
/* 135 */     AnnotationIntrospector ai = cfg.getAnnotationIntrospector();
/* 136 */     AnnotatedClass ac = AnnotatedClass.construct(type.getRawClass(), useAnnotations ? ai : null, r);
/*     */     
/* 138 */     ac.resolveMemberMethods(getSerializationMethodFilter(cfg), false);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 146 */     ac.resolveCreators(true);
/*     */     
/* 148 */     ac.resolveFields(false);
/* 149 */     return new BasicBeanDescription(cfg, type, ac);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public BasicBeanDescription forDeserialization(DeserializationConfig cfg, JavaType type, ClassIntrospector.MixInResolver r)
/*     */   {
/* 156 */     boolean useAnnotations = cfg.isAnnotationProcessingEnabled();
/* 157 */     AnnotationIntrospector ai = cfg.getAnnotationIntrospector();
/* 158 */     AnnotatedClass ac = AnnotatedClass.construct(type.getRawClass(), useAnnotations ? ai : null, r);
/*     */     
/* 160 */     ac.resolveMemberMethods(getDeserializationMethodFilter(cfg), true);
/*     */     
/* 162 */     ac.resolveCreators(true);
/*     */     
/* 164 */     ac.resolveFields(true);
/*     */     
/* 166 */     return new BasicBeanDescription(cfg, type, ac);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public BasicBeanDescription forCreation(DeserializationConfig cfg, JavaType type, ClassIntrospector.MixInResolver r)
/*     */   {
/* 173 */     boolean useAnnotations = cfg.isAnnotationProcessingEnabled();
/* 174 */     AnnotationIntrospector ai = cfg.getAnnotationIntrospector();
/* 175 */     AnnotatedClass ac = AnnotatedClass.construct(type.getRawClass(), useAnnotations ? ai : null, r);
/* 176 */     ac.resolveCreators(true);
/* 177 */     return new BasicBeanDescription(cfg, type, ac);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public BasicBeanDescription forClassAnnotations(MapperConfig<?> cfg, Class<?> c, ClassIntrospector.MixInResolver r)
/*     */   {
/* 184 */     boolean useAnnotations = cfg.isAnnotationProcessingEnabled();
/* 185 */     AnnotationIntrospector ai = cfg.getAnnotationIntrospector();
/* 186 */     AnnotatedClass ac = AnnotatedClass.construct(c, useAnnotations ? ai : null, r);
/* 187 */     return new BasicBeanDescription(cfg, cfg.constructType(c), ac);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public BasicBeanDescription forDirectClassAnnotations(MapperConfig<?> cfg, Class<?> c, ClassIntrospector.MixInResolver r)
/*     */   {
/* 194 */     boolean useAnnotations = cfg.isAnnotationProcessingEnabled();
/* 195 */     AnnotationIntrospector ai = cfg.getAnnotationIntrospector();
/* 196 */     AnnotatedClass ac = AnnotatedClass.constructWithoutSuperTypes(c, useAnnotations ? ai : null, r);
/* 197 */     return new BasicBeanDescription(cfg, cfg.constructType(c), ac);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected MethodFilter getSerializationMethodFilter(SerializationConfig cfg)
/*     */   {
/* 212 */     return DEFAULT_GETTER_FILTER;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected MethodFilter getDeserializationMethodFilter(DeserializationConfig cfg)
/*     */   {
/* 224 */     if (cfg.isEnabled(DeserializationConfig.Feature.USE_GETTERS_AS_SETTERS)) {
/* 225 */       return DEFAULT_SETTER_AND_GETTER_FILTER;
/*     */     }
/*     */     
/* 228 */     return DEFAULT_SETTER_FILTER;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\introspect\BasicClassIntrospector.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */