/*     */ package org.codehaus.janino;
/*     */ 
/*     */ import java.io.BufferedInputStream;
/*     */ import java.io.ByteArrayInputStream;
/*     */ import java.io.File;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.InputStreamReader;
/*     */ import java.io.OutputStream;
/*     */ import java.io.PrintStream;
/*     */ import java.util.ArrayList;
/*     */ import org.codehaus.commons.compiler.CompileException;
/*     */ import org.codehaus.commons.compiler.LocatedException;
/*     */ import org.codehaus.commons.compiler.Location;
/*     */ import org.codehaus.janino.util.Benchmark;
/*     */ import org.codehaus.janino.util.ClassFile;
/*     */ import org.codehaus.janino.util.StringPattern;
/*     */ import org.codehaus.janino.util.resource.DirectoryResourceCreator;
/*     */ import org.codehaus.janino.util.resource.DirectoryResourceFinder;
/*     */ import org.codehaus.janino.util.resource.FileResource;
/*     */ import org.codehaus.janino.util.resource.FileResourceCreator;
/*     */ import org.codehaus.janino.util.resource.PathResourceFinder;
/*     */ import org.codehaus.janino.util.resource.Resource;
/*     */ import org.codehaus.janino.util.resource.ResourceCreator;
/*     */ import org.codehaus.janino.util.resource.ResourceFinder;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class Compiler
/*     */ {
/*     */   private static final boolean DEBUG = false;
/*     */   
/*     */   public static void main(String[] args)
/*     */   {
/*  83 */     File destinationDirectory = NO_DESTINATION_DIRECTORY;
/*  84 */     File[] optionalSourcePath = null;
/*  85 */     File[] classPath = { new File(".") };
/*  86 */     File[] optionalExtDirs = null;
/*  87 */     File[] optionalBootClassPath = null;
/*  88 */     String optionalCharacterEncoding = null;
/*  89 */     boolean verbose = false;
/*  90 */     boolean debugSource = true;
/*  91 */     boolean debugLines = true;
/*  92 */     boolean debugVars = false;
/*  93 */     StringPattern[] warningHandlePatterns = DEFAULT_WARNING_HANDLE_PATTERNS;
/*  94 */     boolean rebuild = false;
/*     */     
/*     */ 
/*     */ 
/*  98 */     for (int i = 0; i < args.length; i++) {
/*  99 */       String arg = args[i];
/* 100 */       if (arg.charAt(0) != '-') break;
/* 101 */       if (arg.equals("-d")) {
/* 102 */         destinationDirectory = new File(args[(++i)]);
/*     */       }
/* 104 */       else if (arg.equals("-sourcepath")) {
/* 105 */         optionalSourcePath = PathResourceFinder.parsePath(args[(++i)]);
/*     */       }
/* 107 */       else if (arg.equals("-classpath")) {
/* 108 */         classPath = PathResourceFinder.parsePath(args[(++i)]);
/*     */       }
/* 110 */       else if (arg.equals("-extdirs")) {
/* 111 */         optionalExtDirs = PathResourceFinder.parsePath(args[(++i)]);
/*     */       }
/* 113 */       else if (arg.equals("-bootclasspath")) {
/* 114 */         optionalBootClassPath = PathResourceFinder.parsePath(args[(++i)]);
/*     */       }
/* 116 */       else if (arg.equals("-encoding")) {
/* 117 */         optionalCharacterEncoding = args[(++i)];
/*     */       }
/* 119 */       else if (arg.equals("-verbose")) {
/* 120 */         verbose = true;
/*     */       }
/* 122 */       else if (arg.equals("-g")) {
/* 123 */         debugSource = debugLines = debugVars = 1;
/*     */       }
/* 125 */       else if (arg.startsWith("-g:")) {
/* 126 */         if (arg.indexOf("none") != -1) debugSource = debugLines = debugVars = 0;
/* 127 */         if (arg.indexOf("source") != -1) debugSource = true;
/* 128 */         if (arg.indexOf("lines") != -1) debugLines = true;
/* 129 */         if (arg.indexOf("vars") != -1) debugVars = true;
/*     */       }
/* 131 */       else if (arg.startsWith("-warn:")) {
/* 132 */         warningHandlePatterns = StringPattern.parseCombinedPattern(arg.substring(6));
/*     */       }
/* 134 */       else if (arg.equals("-rebuild")) {
/* 135 */         rebuild = true;
/*     */       }
/* 137 */       else if (arg.equals("-help")) {
/* 138 */         for (int j = 0; j < USAGE.length; j++) System.out.println(USAGE[j]);
/* 139 */         System.exit(1);
/*     */       } else {
/* 141 */         System.err.println("Unrecognized command line option \"" + arg + "\"; try \"-help\".");
/* 142 */         System.exit(1);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 147 */     if (i == args.length) {
/* 148 */       System.err.println("No source files given on command line; try \"-help\".");
/* 149 */       System.exit(1);
/*     */     }
/* 151 */     File[] sourceFiles = new File[args.length - i];
/* 152 */     for (int j = i; j < args.length; j++) { sourceFiles[(j - i)] = new File(args[j]);
/*     */     }
/*     */     
/* 155 */     Compiler compiler = new Compiler(optionalSourcePath, classPath, optionalExtDirs, optionalBootClassPath, destinationDirectory, optionalCharacterEncoding, verbose, debugSource, debugLines, debugVars, warningHandlePatterns, rebuild);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     try
/*     */     {
/* 172 */       compiler.compile(sourceFiles);
/*     */     } catch (Exception e) {
/* 174 */       System.err.println(e.toString());
/* 175 */       System.exit(1);
/*     */     }
/*     */   }
/*     */   
/* 179 */   private static final String[] USAGE = { "Usage:", "", "  java " + Compiler.class.getName() + " [ <option> ] ... <source-file> ...", "", "Supported <option>s are:", "  -d <output-dir>           Where to save class files", "  -sourcepath <dirlist>     Where to look for other source files", "  -classpath <dirlist>      Where to look for other class files", "  -extdirs <dirlist>        Where to look for other class files", "  -bootclasspath <dirlist>  Where to look for other class files", "  -encoding <encoding>      Encoding of source files, e.g. \"UTF-8\" or \"ISO-8859-1\"", "  -verbose", "  -g                        Generate all debugging info", "  -g:none                   Generate no debugging info", "  -g:{source,lines,vars}    Generate only some debugging info", "  -warn:<pattern-list>      Issue certain warnings; examples:", "    -warn:*                 Enables all warnings", "    -warn:IASF              Only warn against implicit access to static fields", "    -warn:*-IASF            Enables all warnings, except those against implicit", "                            access to static fields", "    -warn:*-IA*+IASF        Enables all warnings, except those against implicit", "                            accesses, but do warn against implicit access to", "                            static fields", "  -rebuild                  Compile all source files, even if the class files", "                            seems up-to-date", "  -help", "", "The default encoding in this environment is \"" + new InputStreamReader(new ByteArrayInputStream(new byte[0])).getEncoding() + "\"." };
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private final ResourceFinder classFileFinder;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 216 */   public static final ResourceFinder FIND_NEXT_TO_SOURCE_FILE = null;
/*     */   
/*     */   private final ResourceCreator classFileCreator;
/* 219 */   public static final ResourceCreator CREATE_NEXT_TO_SOURCE_FILE = null;
/*     */   private final String optionalCharacterEncoding;
/*     */   private final Benchmark benchmark;
/*     */   private final boolean debugSource;
/*     */   private final boolean debugLines;
/*     */   private final boolean debugVars;
/*     */   private final WarningHandler optionalWarningHandler;
/* 226 */   private UnitCompiler.ErrorHandler optionalCompileErrorHandler = null;
/*     */   
/*     */   private final IClassLoader iClassLoader;
/* 229 */   private final ArrayList parsedCompilationUnits = new ArrayList();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Compiler(File[] optionalSourcePath, File[] classPath, File[] optionalExtDirs, File[] optionalBootClassPath, File destinationDirectory, String optionalCharacterEncoding, boolean verbose, boolean debugSource, boolean debugLines, boolean debugVars, StringPattern[] warningHandlePatterns, boolean rebuild)
/*     */   {
/* 288 */     this(new PathResourceFinder(optionalSourcePath == null ? classPath : optionalSourcePath), IClassLoader.createJavacLikePathIClassLoader(optionalBootClassPath, optionalExtDirs, classPath), destinationDirectory == NO_DESTINATION_DIRECTORY ? FIND_NEXT_TO_SOURCE_FILE : rebuild ? ResourceFinder.EMPTY_RESOURCE_FINDER : new DirectoryResourceFinder(destinationDirectory), destinationDirectory == NO_DESTINATION_DIRECTORY ? CREATE_NEXT_TO_SOURCE_FILE : new DirectoryResourceCreator(destinationDirectory), optionalCharacterEncoding, verbose, debugSource, debugLines, debugVars, new FilterWarningHandler(warningHandlePatterns, new SimpleWarningHandler()));
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 318 */     this.benchmark.report("*** JANINO - an embedded compiler for the Java(TM) programming language");
/* 319 */     this.benchmark.report("*** For more information visit http://janino.codehaus.org");
/* 320 */     this.benchmark.report("Source path", optionalSourcePath);
/* 321 */     this.benchmark.report("Class path", classPath);
/* 322 */     this.benchmark.report("Ext dirs", optionalExtDirs);
/* 323 */     this.benchmark.report("Boot class path", optionalBootClassPath);
/* 324 */     this.benchmark.report("Destination directory", destinationDirectory);
/* 325 */     this.benchmark.report("Character encoding", optionalCharacterEncoding);
/* 326 */     this.benchmark.report("Verbose", new Boolean(verbose));
/* 327 */     this.benchmark.report("Debug source", new Boolean(debugSource));
/* 328 */     this.benchmark.report("Debug lines", new Boolean(debugSource));
/* 329 */     this.benchmark.report("Debug vars", new Boolean(debugSource));
/* 330 */     this.benchmark.report("Warning handle patterns", warningHandlePatterns);
/* 331 */     this.benchmark.report("Rebuild", new Boolean(rebuild));
/*     */   }
/*     */   
/* 334 */   public static final File NO_DESTINATION_DIRECTORY = null;
/*     */   
/*     */   public static class SimpleWarningHandler implements WarningHandler {
/* 337 */     public void handleWarning(String handle, String message, Location optionalLocation) { StringBuffer sb = new StringBuffer();
/* 338 */       if (optionalLocation != null) sb.append(optionalLocation).append(": ");
/* 339 */       sb.append("Warning ").append(handle).append(": ").append(message);
/* 340 */       System.err.println(sb.toString());
/*     */     } }
/*     */   
/* 343 */   public static final StringPattern[] DEFAULT_WARNING_HANDLE_PATTERNS = StringPattern.PATTERNS_NONE;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Compiler(ResourceFinder sourceFinder, IClassLoader iClassLoader, ResourceFinder classFileFinder, ResourceCreator classFileCreator, String optionalCharacterEncoding, boolean verbose, boolean debugSource, boolean debugLines, boolean debugVars, WarningHandler optionalWarningHandler)
/*     */   {
/* 375 */     this.classFileFinder = classFileFinder;
/* 376 */     this.classFileCreator = classFileCreator;
/* 377 */     this.optionalCharacterEncoding = optionalCharacterEncoding;
/* 378 */     this.benchmark = new Benchmark(verbose);
/* 379 */     this.debugSource = debugSource;
/* 380 */     this.debugLines = debugLines;
/* 381 */     this.debugVars = debugVars;
/* 382 */     this.optionalWarningHandler = optionalWarningHandler;
/*     */     
/*     */ 
/* 385 */     this.iClassLoader = new CompilerIClassLoader(sourceFinder, iClassLoader);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setCompileErrorHandler(UnitCompiler.ErrorHandler optionalCompileErrorHandler)
/*     */   {
/* 408 */     this.optionalCompileErrorHandler = optionalCompileErrorHandler;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean compile(File[] sourceFiles)
/*     */     throws CompileException, IOException
/*     */   {
/* 443 */     this.benchmark.report("Source files", sourceFiles);
/*     */     
/* 445 */     Resource[] sourceFileResources = new Resource[sourceFiles.length];
/* 446 */     for (int i = 0; i < sourceFiles.length; i++) sourceFileResources[i] = new FileResource(sourceFiles[i]);
/* 447 */     compile(sourceFileResources);
/* 448 */     return true;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean compile(Resource[] sourceResources)
/*     */     throws CompileException, IOException
/*     */   {
/* 462 */     UnitCompiler.ErrorHandler ceh = this.optionalCompileErrorHandler != null ? this.optionalCompileErrorHandler : new UnitCompiler.ErrorHandler()
/*     */     {
/*     */ 
/*     */ 
/* 466 */       int compileErrorCount = 0;
/*     */       
/* 468 */       public void handleError(String message, Location optionalLocation) throws CompileException { CompileException ex = new CompileException(message, optionalLocation);
/* 469 */         if (++this.compileErrorCount >= 20) throw ex;
/* 470 */         System.err.println(ex.getMessage());
/*     */       }
/*     */       
/*     */ 
/* 474 */     };
/* 475 */     this.benchmark.beginReporting();
/*     */     
/*     */     try
/*     */     {
/* 479 */       this.parsedCompilationUnits.clear();
/* 480 */       for (int i = 0; i < sourceResources.length; i++)
/*     */       {
/* 482 */         this.parsedCompilationUnits.add(new UnitCompiler(parseCompilationUnit(sourceResources[i].getFileName(), new BufferedInputStream(sourceResources[i].open()), this.optionalCharacterEncoding), this.iClassLoader));
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 492 */       for (int i = 0; i < this.parsedCompilationUnits.size(); i++) {
/* 493 */         UnitCompiler unitCompiler = (UnitCompiler)this.parsedCompilationUnits.get(i);
/* 494 */         Java.CompilationUnit cu = unitCompiler.compilationUnit;
/* 495 */         if (cu.optionalFileName == null) throw new JaninoRuntimeException();
/* 496 */         File sourceFile = new File(cu.optionalFileName);
/*     */         
/* 498 */         unitCompiler.setCompileErrorHandler(ceh);
/* 499 */         unitCompiler.setWarningHandler(this.optionalWarningHandler);
/*     */         
/* 501 */         this.benchmark.beginReporting("Compiling compilation unit \"" + sourceFile + "\"");
/*     */         
/*     */         ClassFile[] classFiles;
/*     */         try
/*     */         {
/* 506 */           classFiles = unitCompiler.compileUnit(this.debugSource, this.debugLines, this.debugVars);
/*     */         } finally {
/* 508 */           this.benchmark.endReporting();
/*     */         }
/*     */         
/*     */ 
/* 512 */         this.benchmark.beginReporting("Storing " + classFiles.length + " class file(s) resulting from compilation unit \"" + sourceFile + "\"");
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         try
/*     */         {
/* 520 */           for (int j = 0; j < classFiles.length; j++) {
/* 521 */             storeClassFile(classFiles[j], sourceFile);
/*     */           }
/*     */         } finally {
/* 524 */           this.benchmark.endReporting();
/*     */         }
/*     */       }
/*     */     } finally {
/* 528 */       this.benchmark.endReporting("Compiled " + this.parsedCompilationUnits.size() + " compilation unit(s)");
/*     */     }
/* 530 */     return true;
/*     */   }
/*     */   
/*     */   /* Error */
/*     */   private Java.CompilationUnit parseCompilationUnit(String fileName, InputStream inputStream, String optionalCharacterEncoding)
/*     */     throws CompileException, IOException
/*     */   {
/*     */     // Byte code:
/*     */     //   0: new 140	org/codehaus/janino/Scanner
/*     */     //   3: dup
/*     */     //   4: aload_1
/*     */     //   5: aload_2
/*     */     //   6: aload_3
/*     */     //   7: invokespecial 141	org/codehaus/janino/Scanner:<init>	(Ljava/lang/String;Ljava/io/InputStream;Ljava/lang/String;)V
/*     */     //   10: astore 4
/*     */     //   12: aload 4
/*     */     //   14: aload_0
/*     */     //   15: getfield 100	org/codehaus/janino/Compiler:optionalWarningHandler	Lorg/codehaus/janino/WarningHandler;
/*     */     //   18: invokevirtual 142	org/codehaus/janino/Scanner:setWarningHandler	(Lorg/codehaus/janino/WarningHandler;)V
/*     */     //   21: new 143	org/codehaus/janino/Parser
/*     */     //   24: dup
/*     */     //   25: aload 4
/*     */     //   27: invokespecial 144	org/codehaus/janino/Parser:<init>	(Lorg/codehaus/janino/Scanner;)V
/*     */     //   30: astore 5
/*     */     //   32: aload 5
/*     */     //   34: aload_0
/*     */     //   35: getfield 100	org/codehaus/janino/Compiler:optionalWarningHandler	Lorg/codehaus/janino/WarningHandler;
/*     */     //   38: invokevirtual 145	org/codehaus/janino/Parser:setWarningHandler	(Lorg/codehaus/janino/WarningHandler;)V
/*     */     //   41: aload_0
/*     */     //   42: getfield 1	org/codehaus/janino/Compiler:benchmark	Lorg/codehaus/janino/util/Benchmark;
/*     */     //   45: new 45	java/lang/StringBuffer
/*     */     //   48: dup
/*     */     //   49: invokespecial 46	java/lang/StringBuffer:<init>	()V
/*     */     //   52: ldc -110
/*     */     //   54: invokevirtual 48	java/lang/StringBuffer:append	(Ljava/lang/String;)Ljava/lang/StringBuffer;
/*     */     //   57: aload_1
/*     */     //   58: invokevirtual 48	java/lang/StringBuffer:append	(Ljava/lang/String;)Ljava/lang/StringBuffer;
/*     */     //   61: ldc -127
/*     */     //   63: invokevirtual 48	java/lang/StringBuffer:append	(Ljava/lang/String;)Ljava/lang/StringBuffer;
/*     */     //   66: invokevirtual 50	java/lang/StringBuffer:toString	()Ljava/lang/String;
/*     */     //   69: invokevirtual 130	org/codehaus/janino/util/Benchmark:beginReporting	(Ljava/lang/String;)V
/*     */     //   72: aload 5
/*     */     //   74: invokevirtual 147	org/codehaus/janino/Parser:parseCompilationUnit	()Lorg/codehaus/janino/Java$CompilationUnit;
/*     */     //   77: astore 6
/*     */     //   79: aload_0
/*     */     //   80: getfield 1	org/codehaus/janino/Compiler:benchmark	Lorg/codehaus/janino/util/Benchmark;
/*     */     //   83: invokevirtual 132	org/codehaus/janino/util/Benchmark:endReporting	()V
/*     */     //   86: aload_2
/*     */     //   87: invokevirtual 148	java/io/InputStream:close	()V
/*     */     //   90: aload 6
/*     */     //   92: areturn
/*     */     //   93: astore 7
/*     */     //   95: aload_0
/*     */     //   96: getfield 1	org/codehaus/janino/Compiler:benchmark	Lorg/codehaus/janino/util/Benchmark;
/*     */     //   99: invokevirtual 132	org/codehaus/janino/util/Benchmark:endReporting	()V
/*     */     //   102: aload 7
/*     */     //   104: athrow
/*     */     //   105: astore 8
/*     */     //   107: aload_2
/*     */     //   108: invokevirtual 148	java/io/InputStream:close	()V
/*     */     //   111: aload 8
/*     */     //   113: athrow
/*     */     // Line number table:
/*     */     //   Java source line #545	-> byte code offset #0
/*     */     //   Java source line #546	-> byte code offset #12
/*     */     //   Java source line #547	-> byte code offset #21
/*     */     //   Java source line #548	-> byte code offset #32
/*     */     //   Java source line #550	-> byte code offset #41
/*     */     //   Java source line #552	-> byte code offset #72
/*     */     //   Java source line #554	-> byte code offset #79
/*     */     //   Java source line #557	-> byte code offset #86
/*     */     //   Java source line #554	-> byte code offset #93
/*     */     //   Java source line #557	-> byte code offset #105
/*     */     // Local variable table:
/*     */     //   start	length	slot	name	signature
/*     */     //   0	114	0	this	Compiler
/*     */     //   0	114	1	fileName	String
/*     */     //   0	114	2	inputStream	InputStream
/*     */     //   0	114	3	optionalCharacterEncoding	String
/*     */     //   10	16	4	scanner	Scanner
/*     */     //   30	43	5	parser	Parser
/*     */     //   77	14	6	localCompilationUnit	Java.CompilationUnit
/*     */     //   93	10	7	localObject1	Object
/*     */     //   105	7	8	localObject2	Object
/*     */     // Exception table:
/*     */     //   from	to	target	type
/*     */     //   72	79	93	finally
/*     */     //   93	95	93	finally
/*     */     //   0	86	105	finally
/*     */     //   93	107	105	finally
/*     */   }
/*     */   
/*     */   public static File getClassFile(String className, File sourceFile, File optionalDestinationDirectory)
/*     */   {
/* 578 */     if (optionalDestinationDirectory != null) {
/* 579 */       return new File(optionalDestinationDirectory, ClassFile.getClassFileResourceName(className));
/*     */     }
/* 581 */     int idx = className.lastIndexOf('.');
/* 582 */     return new File(sourceFile.getParentFile(), ClassFile.getClassFileResourceName(className.substring(idx + 1)));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void storeClassFile(ClassFile classFile, File sourceFile)
/*     */     throws IOException
/*     */   {
/* 596 */     String classFileResourceName = ClassFile.getClassFileResourceName(classFile.getThisClassName());
/*     */     
/*     */     ResourceCreator rc;
/*     */     ResourceCreator rc;
/* 600 */     if (this.classFileCreator != CREATE_NEXT_TO_SOURCE_FILE) {
/* 601 */       rc = this.classFileCreator;
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/* 606 */       rc = new FileResourceCreator() { private final File val$sourceFile;
/*     */         
/* 608 */         protected File getFile(String resourceName) { return new File(this.val$sourceFile.getParentFile(), resourceName.substring(resourceName.lastIndexOf('/') + 1)); }
/*     */       };
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 615 */     OutputStream os = rc.createResource(classFileResourceName);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     try
/*     */     {
/* 631 */       classFile.store(os); return;
/*     */     } catch (IOException ex) {
/*     */       try {
/* 634 */         os.close(); } catch (IOException e) {}
/* 635 */       os = null;
/* 636 */       if (!rc.deleteResource(classFileResourceName)) {
/* 637 */         throw new IOException("Could not delete incompletely written class file \"" + classFileResourceName + "\"");
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 643 */       throw ex;
/*     */     } finally {
/* 645 */       if (os != null) { try { os.close();
/*     */         }
/*     */         catch (IOException e) {}
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private class CompilerIClassLoader
/*     */     extends IClassLoader
/*     */   {
/*     */     private final ResourceFinder sourceFinder;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public CompilerIClassLoader(ResourceFinder sourceFinder, IClassLoader optionalParentIClassLoader)
/*     */     {
/* 672 */       super();
/* 673 */       this.sourceFinder = sourceFinder;
/* 674 */       super.postConstruct();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected IClass findIClass(String type)
/*     */       throws ClassNotFoundException
/*     */     {
/* 686 */       String className = Descriptor.toClassName(type);
/*     */       
/*     */ 
/*     */ 
/* 690 */       if (className.startsWith("java.")) { return null;
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 695 */       int idx = className.indexOf('$');
/* 696 */       String topLevelClassName = idx == -1 ? className : className.substring(0, idx);
/*     */       
/*     */ 
/*     */ 
/* 700 */       for (int i = 0; i < Compiler.this.parsedCompilationUnits.size(); i++) {
/* 701 */         UnitCompiler uc = (UnitCompiler)Compiler.this.parsedCompilationUnits.get(i);
/* 702 */         IClass res = uc.findClass(topLevelClassName);
/* 703 */         if (res != null) {
/* 704 */           if (!className.equals(topLevelClassName)) {
/* 705 */             res = uc.findClass(className);
/* 706 */             if (res == null) return null;
/*     */           }
/* 708 */           defineIClass(res);
/* 709 */           return res;
/*     */         }
/*     */       }
/*     */       
/*     */ 
/* 714 */       Resource sourceResource = this.sourceFinder.findResource(ClassFile.getSourceResourceName(className));
/* 715 */       if (sourceResource == null) { return null;
/*     */       }
/*     */       Resource classFileResource;
/*     */       Resource classFileResource;
/* 719 */       if (Compiler.this.classFileFinder != Compiler.FIND_NEXT_TO_SOURCE_FILE) {
/* 720 */         classFileResource = Compiler.this.classFileFinder.findResource(ClassFile.getClassFileResourceName(className));
/*     */       }
/*     */       else
/*     */       {
/* 724 */         if (!(sourceResource instanceof FileResource)) return null;
/* 725 */         File classFile = new File(((FileResource)sourceResource).getFile().getParentFile(), ClassFile.getClassFileResourceName(className.substring(className.lastIndexOf('.') + 1)));
/*     */         
/*     */ 
/*     */ 
/* 729 */         classFileResource = classFile.exists() ? new FileResource(classFile) : null;
/*     */       }
/*     */       
/*     */ 
/* 733 */       if ((classFileResource != null) && (sourceResource.lastModified() <= classFileResource.lastModified()))
/*     */       {
/*     */ 
/* 736 */         return defineIClassFromClassFileResource(classFileResource);
/*     */       }
/*     */       
/*     */ 
/* 740 */       return defineIClassFromSourceResource(sourceResource, className);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     private IClass defineIClassFromSourceResource(Resource sourceResource, String className)
/*     */       throws ClassNotFoundException
/*     */     {
/*     */       UnitCompiler uc;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       try
/*     */       {
/* 760 */         Java.CompilationUnit cu = Compiler.this.parseCompilationUnit(sourceResource.getFileName(), new BufferedInputStream(sourceResource.open()), Compiler.this.optionalCharacterEncoding);
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 765 */         uc = new UnitCompiler(cu, Compiler.this.iClassLoader);
/*     */       } catch (IOException ex) {
/* 767 */         throw new ClassNotFoundException("Parsing compilation unit \"" + sourceResource + "\"", ex);
/*     */       } catch (CompileException ex) {
/* 769 */         throw new ClassNotFoundException("Parsing compilation unit \"" + sourceResource + "\"", ex);
/*     */       }
/*     */       
/*     */ 
/* 773 */       Compiler.this.parsedCompilationUnits.add(uc);
/*     */       
/*     */ 
/* 776 */       IClass res = uc.findClass(className);
/* 777 */       if (res == null)
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 784 */         return null;
/*     */       }
/* 786 */       defineIClass(res);
/* 787 */       return res;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     private IClass defineIClassFromClassFileResource(Resource classFileResource)
/*     */       throws ClassNotFoundException
/*     */     {
/* 795 */       Compiler.this.benchmark.beginReporting("Loading class file \"" + classFileResource.getFileName() + "\"");
/*     */       try {
/* 797 */         InputStream is = null;
/*     */         try
/*     */         {
/* 800 */           is = classFileResource.open();
/* 801 */           ClassFile cf = new ClassFile(new BufferedInputStream(is));
/*     */           
/*     */ 
/*     */ 
/* 805 */           if (is != null) try { is.close();
/*     */             } catch (IOException e) {}
/* 807 */           result = new ClassFileIClass(cf, this);
/*     */         }
/*     */         catch (IOException ex)
/*     */         {
/* 803 */           throw new ClassNotFoundException("Opening class file resource \"" + classFileResource + "\"", ex);
/*     */         } finally {
/* 805 */           if (is != null) { try { is.close();
/*     */             }
/*     */             catch (IOException e) {}
/*     */           }
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */         ClassFileIClass result;
/*     */         
/*     */ 
/* 816 */         defineIClass(result);
/* 817 */         result.resolveAllClasses();
/*     */         
/* 819 */         return result;
/*     */       } finally {
/* 821 */         Compiler.this.benchmark.endReporting();
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\Compiler.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */