/*     */ package org.hibernate.annotations.common.reflection.java;
/*     */ 
/*     */ import java.lang.reflect.AnnotatedElement;
/*     */ import java.lang.reflect.Member;
/*     */ import java.lang.reflect.Method;
/*     */ import java.lang.reflect.ParameterizedType;
/*     */ import java.lang.reflect.Type;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import org.hibernate.annotations.common.reflection.AnnotationReader;
/*     */ import org.hibernate.annotations.common.reflection.ReflectionManager;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.annotations.common.reflection.XMethod;
/*     */ import org.hibernate.annotations.common.reflection.XPackage;
/*     */ import org.hibernate.annotations.common.reflection.XProperty;
/*     */ import org.hibernate.annotations.common.reflection.java.generics.IdentityTypeEnvironment;
/*     */ import org.hibernate.annotations.common.reflection.java.generics.TypeEnvironment;
/*     */ import org.hibernate.annotations.common.reflection.java.generics.TypeEnvironmentFactory;
/*     */ import org.hibernate.annotations.common.reflection.java.generics.TypeSwitch;
/*     */ import org.hibernate.annotations.common.reflection.java.generics.TypeUtils;
/*     */ import org.hibernate.annotations.common.util.ReflectHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class JavaReflectionManager
/*     */   implements ReflectionManager
/*     */ {
/*     */   private static class TypeKey
/*     */     extends Pair<Type, TypeEnvironment>
/*     */   {
/*     */     TypeKey(Type t, TypeEnvironment context)
/*     */     {
/*  35 */       super(context);
/*     */     }
/*     */   }
/*     */   
/*     */   private static class MemberKey extends Pair<Member, TypeEnvironment> {
/*     */     MemberKey(Member member, TypeEnvironment context) {
/*  41 */       super(context);
/*     */     }
/*     */   }
/*     */   
/*  45 */   private final Map<TypeKey, JavaXClass> xClasses = new HashMap();
/*     */   
/*  47 */   private final Map<Package, JavaXPackage> packagesToXPackages = new HashMap();
/*     */   
/*  49 */   private final Map<MemberKey, JavaXProperty> xProperties = new HashMap();
/*     */   
/*  51 */   private final Map<MemberKey, JavaXMethod> xMethods = new HashMap();
/*     */   
/*  53 */   private final TypeEnvironmentFactory typeEnvs = new TypeEnvironmentFactory();
/*     */   
/*     */   public XClass toXClass(Class clazz) {
/*  56 */     return toXClass(clazz, IdentityTypeEnvironment.INSTANCE);
/*     */   }
/*     */   
/*     */   public Class toClass(XClass xClazz) {
/*  60 */     if (!(xClazz instanceof JavaXClass)) {
/*  61 */       throw new IllegalArgumentException("XClass not coming from this ReflectionManager implementation");
/*     */     }
/*  63 */     return (Class)((JavaXClass)xClazz).toAnnotatedElement();
/*     */   }
/*     */   
/*     */   public Method toMethod(XMethod xMethod) {
/*  67 */     if (!(xMethod instanceof JavaXMethod)) {
/*  68 */       throw new IllegalArgumentException("XMethod not coming from this ReflectionManager implementation");
/*     */     }
/*  70 */     return (Method)((JavaXAnnotatedElement)xMethod).toAnnotatedElement();
/*     */   }
/*     */   
/*     */   public XClass classForName(String name, Class caller) throws ClassNotFoundException {
/*  74 */     return toXClass(ReflectHelper.classForName(name, caller));
/*     */   }
/*     */   
/*     */   public XPackage packageForName(String packageName) throws ClassNotFoundException {
/*  78 */     return getXAnnotatedElement(ReflectHelper.classForName(packageName + ".package-info").getPackage());
/*     */   }
/*     */   
/*     */   XClass toXClass(Type t, final TypeEnvironment context) {
/*  82 */     (XClass)new TypeSwitch()
/*     */     {
/*     */       public XClass caseClass(Class classType) {
/*  85 */         JavaReflectionManager.TypeKey key = new JavaReflectionManager.TypeKey(classType, context);
/*  86 */         JavaXClass result = (JavaXClass)JavaReflectionManager.this.xClasses.get(key);
/*  87 */         if (result == null) {
/*  88 */           result = new JavaXClass(classType, context, JavaReflectionManager.this);
/*  89 */           JavaReflectionManager.this.xClasses.put(key, result);
/*     */         }
/*  91 */         return result;
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*  96 */       public XClass caseParameterizedType(ParameterizedType parameterizedType) { return JavaReflectionManager.this.toXClass(parameterizedType.getRawType(), JavaReflectionManager.this.typeEnvs.getEnvironment(parameterizedType, context)); } }.doSwitch(context.bind(t));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   XPackage getXAnnotatedElement(Package pkg)
/*     */   {
/* 104 */     JavaXPackage xPackage = (JavaXPackage)this.packagesToXPackages.get(pkg);
/* 105 */     if (xPackage == null) {
/* 106 */       xPackage = new JavaXPackage(pkg, this);
/* 107 */       this.packagesToXPackages.put(pkg, xPackage);
/*     */     }
/* 109 */     return xPackage;
/*     */   }
/*     */   
/*     */   XProperty getXProperty(Member member, TypeEnvironment context) {
/* 113 */     MemberKey key = new MemberKey(member, context);
/*     */     
/* 115 */     JavaXProperty xProperty = (JavaXProperty)this.xProperties.get(key);
/* 116 */     if (xProperty == null) {
/* 117 */       xProperty = JavaXProperty.create(member, context, this);
/* 118 */       this.xProperties.put(key, xProperty);
/*     */     }
/* 120 */     return xProperty;
/*     */   }
/*     */   
/*     */   XMethod getXMethod(Member member, TypeEnvironment context) {
/* 124 */     MemberKey key = new MemberKey(member, context);
/*     */     
/* 126 */     JavaXMethod xMethod = (JavaXMethod)this.xMethods.get(key);
/* 127 */     if (xMethod == null) {
/* 128 */       xMethod = JavaXMethod.create(member, context, this);
/* 129 */       this.xMethods.put(key, xMethod);
/*     */     }
/* 131 */     return xMethod;
/*     */   }
/*     */   
/*     */   TypeEnvironment getTypeEnvironment(Type t) {
/* 135 */     (TypeEnvironment)new TypeSwitch()
/*     */     {
/*     */       public TypeEnvironment caseClass(Class classType) {
/* 138 */         return JavaReflectionManager.this.typeEnvs.getEnvironment(classType);
/*     */       }
/*     */       
/*     */       public TypeEnvironment caseParameterizedType(ParameterizedType parameterizedType)
/*     */       {
/* 143 */         return JavaReflectionManager.this.typeEnvs.getEnvironment(parameterizedType);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 148 */       public TypeEnvironment defaultCase(Type type) { return IdentityTypeEnvironment.INSTANCE; } }.doSwitch(t);
/*     */   }
/*     */   
/*     */ 
/*     */   public JavaXType toXType(TypeEnvironment context, Type propType)
/*     */   {
/* 154 */     Type boundType = toApproximatingEnvironment(context).bind(propType);
/* 155 */     if (TypeUtils.isArray(boundType)) {
/* 156 */       return new JavaXArrayType(propType, context, this);
/*     */     }
/* 158 */     if (TypeUtils.isCollection(boundType)) {
/* 159 */       return new JavaXCollectionType(propType, context, this);
/*     */     }
/* 161 */     if (TypeUtils.isSimple(boundType)) {
/* 162 */       return new JavaXSimpleType(propType, context, this);
/*     */     }
/* 164 */     throw new IllegalArgumentException("No PropertyTypeExtractor available for type void ");
/*     */   }
/*     */   
/*     */   public boolean equals(XClass class1, Class class2) {
/* 168 */     if (class1 == null) {
/* 169 */       return class2 == null;
/*     */     }
/* 171 */     return ((JavaXClass)class1).toClass().equals(class2);
/*     */   }
/*     */   
/*     */   public TypeEnvironment toApproximatingEnvironment(TypeEnvironment context) {
/* 175 */     return this.typeEnvs.toApproximatingEnvironment(context);
/*     */   }
/*     */   
/*     */   public AnnotationReader buildAnnotationReader(AnnotatedElement annotatedElement) {
/* 179 */     return new JavaAnnotationReader(annotatedElement);
/*     */   }
/*     */   
/*     */   public Map getDefaults() {
/* 183 */     return new HashMap();
/*     */   }
/*     */ }


/* Location:              /Users/anandkum/Downloads/hibernate-commons-annotations-3.0.0.ga.jar!/org/hibernate/annotations/common/reflection/java/JavaReflectionManager.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */