package prefuse.activity;

import java.util.ArrayList;
import prefuse.util.PrefuseConfig;

public class ActivityManager
  extends Thread
{
  private static ActivityManager s_instance;
  private ArrayList m_activities = new ArrayList();
  private ArrayList m_tmp = new ArrayList();
  private long m_nextTime = Long.MAX_VALUE;
  private boolean m_run;
  
  private static synchronized ActivityManager getInstance()
  {
    if ((s_instance == null) || (!s_instance.isAlive())) {
      s_instance = new ActivityManager();
    }
    return s_instance;
  }
  
  private ActivityManager()
  {
    super("prefuse_ActivityManager");
    int i = PrefuseConfig.getInt("activity.threadPriority");
    if ((i >= 1) && (i <= 10)) {
      setPriority(i);
    }
    setDaemon(true);
    start();
  }
  
  public static void stopThread()
  {
    ActivityManager localActivityManager;
    synchronized (ActivityManager.class)
    {
      localActivityManager = s_instance;
    }
    if (localActivityManager != null) {
      localActivityManager._stop();
    }
  }
  
  static void schedule(Activity paramActivity)
  {
    getInstance()._schedule(paramActivity, paramActivity.getStartTime());
  }
  
  static void scheduleNow(Activity paramActivity)
  {
    getInstance()._schedule(paramActivity, System.currentTimeMillis());
  }
  
  static void scheduleAt(Activity paramActivity, long paramLong)
  {
    getInstance()._schedule(paramActivity, paramLong);
  }
  
  static void scheduleAfter(Activity paramActivity1, Activity paramActivity2)
  {
    getInstance()._scheduleAfter(paramActivity1, paramActivity2);
  }
  
  static void alwaysScheduleAfter(Activity paramActivity1, Activity paramActivity2)
  {
    getInstance()._alwaysScheduleAfter(paramActivity1, paramActivity2);
  }
  
  static void removeActivity(Activity paramActivity)
  {
    getInstance()._removeActivity(paramActivity);
  }
  
  public static int activityCount()
  {
    return getInstance()._activityCount();
  }
  
  private synchronized void _stop()
  {
    while (this.m_activities.size() > 0)
    {
      Activity localActivity = (Activity)this.m_activities.get(this.m_activities.size() - 1);
      localActivity.cancel();
    }
    _setRunning(false);
    notify();
  }
  
  private void _schedule(Activity paramActivity, long paramLong)
  {
    if (paramActivity.isScheduled()) {
      return;
    }
    paramActivity.setStartTime(paramLong);
    synchronized (this)
    {
      this.m_activities.add(paramActivity);
      paramActivity.setScheduled(true);
      if (paramLong < this.m_nextTime)
      {
        this.m_nextTime = paramLong;
        notify();
      }
    }
  }
  
  private void _scheduleAfter(Activity paramActivity1, Activity paramActivity2)
  {
    paramActivity1.addActivityListener(new ScheduleAfterActivity(paramActivity2, true));
  }
  
  private void _alwaysScheduleAfter(Activity paramActivity1, Activity paramActivity2)
  {
    paramActivity1.addActivityListener(new ScheduleAfterActivity(paramActivity2, false));
  }
  
  private boolean _removeActivity(Activity paramActivity)
  {
    boolean bool;
    synchronized (this)
    {
      bool = this.m_activities.remove(paramActivity);
      if ((bool) && (this.m_activities.size() == 0)) {
        this.m_nextTime = Long.MAX_VALUE;
      }
    }
    if (bool) {
      paramActivity.setScheduled(false);
    }
    return bool;
  }
  
  private synchronized int _activityCount()
  {
    return this.m_activities.size();
  }
  
  private synchronized void _setRunning(boolean paramBoolean)
  {
    this.m_run = paramBoolean;
  }
  
  private synchronized boolean _keepRunning()
  {
    return this.m_run;
  }
  
  public void run()
  {
    _setRunning(true);
    while (_keepRunning()) {
      if (_activityCount() > 0)
      {
        long l1 = System.currentTimeMillis();
        long l2 = -1L;
        synchronized (this)
        {
          for (int i = 0; i < this.m_activities.size(); i++)
          {
            Activity localActivity2 = (Activity)this.m_activities.get(i);
            this.m_tmp.add(localActivity2);
            if (l1 >= localActivity2.getStopTime())
            {
              this.m_activities.remove(i--);
              localActivity2.setScheduled(false);
            }
          }
          if (this.m_activities.size() == 0) {
            this.m_nextTime = Long.MAX_VALUE;
          }
        }
        for (??? = 0; ??? < this.m_tmp.size(); ???++)
        {
          Activity localActivity1 = (Activity)this.m_tmp.get(???);
          long l3 = localActivity1.runActivity(l1);
          l2 = l2 < 0L ? l3 : l3 < 0L ? l2 : Math.min(l2, l3);
        }
        this.m_tmp.clear();
        if (l2 != -1L) {
          try
          {
            synchronized (this)
            {
              wait(l2);
            }
          }
          catch (InterruptedException localInterruptedException2) {}
        }
      }
      else
      {
        try
        {
          synchronized (this)
          {
            wait();
          }
        }
        catch (InterruptedException localInterruptedException1) {}
      }
    }
  }
  
  public class ScheduleAfterActivity
    extends ActivityAdapter
  {
    Activity after;
    boolean remove;
    
    public ScheduleAfterActivity(Activity paramActivity, boolean paramBoolean)
    {
      this.after = paramActivity;
      this.remove = paramBoolean;
    }
    
    public void activityFinished(Activity paramActivity)
    {
      if (this.remove) {
        paramActivity.removeActivityListener(this);
      }
      ActivityManager.scheduleNow(this.after);
    }
    
    public void activityCancelled(Activity paramActivity)
    {
      if (this.remove) {
        paramActivity.removeActivityListener(this);
      }
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\activity\ActivityManager.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */