package prefuse.data.column;

import java.util.BitSet;
import java.util.Iterator;
import java.util.Set;
import prefuse.data.DataTypeException;
import prefuse.data.Table;
import prefuse.data.event.ColumnListener;
import prefuse.data.event.ExpressionListener;
import prefuse.data.expression.Expression;
import prefuse.data.expression.ExpressionAnalyzer;

public class ExpressionColumn
  extends AbstractColumn
{
  private Expression m_expr;
  private Table m_table;
  private Set m_columns;
  private BitSet m_valid;
  private Column m_cache;
  private Listener m_lstnr;
  
  public ExpressionColumn(Table paramTable, Expression paramExpression)
  {
    super(paramExpression.getType(paramTable.getSchema()));
    this.m_table = paramTable;
    this.m_expr = paramExpression;
    this.m_lstnr = new Listener(null);
    init();
    int i = this.m_table.getRowCount();
    this.m_cache = ColumnFactory.getColumn(getColumnClass(), i);
    this.m_valid = new BitSet(i);
    this.m_expr.addExpressionListener(this.m_lstnr);
  }
  
  protected void init()
  {
    String str;
    Column localColumn;
    if ((this.m_columns != null) && (this.m_columns.size() > 0))
    {
      localIterator = this.m_columns.iterator();
      while (localIterator.hasNext())
      {
        str = (String)localIterator.next();
        localColumn = this.m_table.getColumn(str);
        localColumn.removeColumnListener(this.m_lstnr);
      }
    }
    this.m_columns = ExpressionAnalyzer.getReferencedColumns(this.m_expr);
    Iterator localIterator = this.m_columns.iterator();
    while (localIterator.hasNext())
    {
      str = (String)localIterator.next();
      if (this.m_table.getColumn(str) == null) {
        throw new IllegalArgumentException("Table must contain all columns referenced by the expression. Bad column name: " + str);
      }
    }
    localIterator = this.m_columns.iterator();
    while (localIterator.hasNext())
    {
      str = (String)localIterator.next();
      localColumn = this.m_table.getColumn(str);
      localColumn.addColumnListener(this.m_lstnr);
    }
  }
  
  public int getRowCount()
  {
    return this.m_cache.getRowCount();
  }
  
  public void setMaximumRow(int paramInt)
  {
    this.m_cache.setMaximumRow(paramInt);
  }
  
  public boolean isCacheValid(int paramInt)
  {
    return this.m_valid.get(paramInt);
  }
  
  public void invalidateCache(int paramInt1, int paramInt2)
  {
    this.m_valid.clear(paramInt1, paramInt2 + 1);
  }
  
  public void revertToDefault(int paramInt) {}
  
  public boolean canSet(Class paramClass)
  {
    return false;
  }
  
  public Object get(int paramInt)
  {
    rangeCheck(paramInt);
    if (isCacheValid(paramInt)) {
      return this.m_cache.get(paramInt);
    }
    Object localObject = this.m_expr.get(this.m_table.getTuple(paramInt));
    if (this.m_cache.canSet(localObject.getClass()))
    {
      this.m_cache.set(localObject, paramInt);
      this.m_valid.set(paramInt);
    }
    return localObject;
  }
  
  public void set(Object paramObject, int paramInt)
    throws DataTypeException
  {
    throw new UnsupportedOperationException();
  }
  
  private void rangeCheck(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= getRowCount())) {
      throw new IndexOutOfBoundsException();
    }
  }
  
  public boolean getBoolean(int paramInt)
    throws DataTypeException
  {
    if (!canGetBoolean()) {
      throw new DataTypeException(Boolean.TYPE);
    }
    rangeCheck(paramInt);
    if (isCacheValid(paramInt)) {
      return this.m_cache.getBoolean(paramInt);
    }
    boolean bool = this.m_expr.getBoolean(this.m_table.getTuple(paramInt));
    this.m_cache.setBoolean(bool, paramInt);
    this.m_valid.set(paramInt);
    return bool;
  }
  
  public int getInt(int paramInt)
    throws DataTypeException
  {
    if (!canGetInt()) {
      throw new DataTypeException(Integer.TYPE);
    }
    rangeCheck(paramInt);
    if (isCacheValid(paramInt)) {
      return this.m_cache.getInt(paramInt);
    }
    int i = this.m_expr.getInt(this.m_table.getTuple(paramInt));
    this.m_cache.setInt(i, paramInt);
    this.m_valid.set(paramInt);
    return i;
  }
  
  public double getDouble(int paramInt)
    throws DataTypeException
  {
    if (!canGetDouble()) {
      throw new DataTypeException(Double.TYPE);
    }
    rangeCheck(paramInt);
    if (isCacheValid(paramInt)) {
      return this.m_cache.getDouble(paramInt);
    }
    double d = this.m_expr.getDouble(this.m_table.getTuple(paramInt));
    this.m_cache.setDouble(d, paramInt);
    this.m_valid.set(paramInt);
    return d;
  }
  
  public float getFloat(int paramInt)
    throws DataTypeException
  {
    if (!canGetFloat()) {
      throw new DataTypeException(Float.TYPE);
    }
    rangeCheck(paramInt);
    if (isCacheValid(paramInt)) {
      return this.m_cache.getFloat(paramInt);
    }
    float f = this.m_expr.getFloat(this.m_table.getTuple(paramInt));
    this.m_cache.setFloat(f, paramInt);
    this.m_valid.set(paramInt);
    return f;
  }
  
  public long getLong(int paramInt)
    throws DataTypeException
  {
    if (!canGetLong()) {
      throw new DataTypeException(Long.TYPE);
    }
    rangeCheck(paramInt);
    if (isCacheValid(paramInt)) {
      return this.m_cache.getLong(paramInt);
    }
    long l = this.m_expr.getLong(this.m_table.getTuple(paramInt));
    this.m_cache.setLong(l, paramInt);
    this.m_valid.set(paramInt);
    return l;
  }
  
  private class Listener
    implements ColumnListener, ExpressionListener
  {
    private Listener() {}
    
    public void columnChanged(int paramInt1, int paramInt2)
    {
      if ((paramInt1 == paramInt2) && (ExpressionColumn.this.isCacheValid(paramInt1)))
      {
        if (!ExpressionColumn.this.m_table.isValidRow(paramInt1)) {
          return;
        }
        ExpressionColumn.this.invalidateCache(paramInt1, paramInt2);
        Class localClass = ExpressionColumn.this.getColumnClass();
        if (Integer.TYPE == localClass) {
          ExpressionColumn.this.fireColumnEvent(paramInt1, ExpressionColumn.this.m_cache.getInt(paramInt1));
        } else if (Long.TYPE == localClass) {
          ExpressionColumn.this.fireColumnEvent(paramInt1, ExpressionColumn.this.m_cache.getLong(paramInt1));
        } else if (Float.TYPE == localClass) {
          ExpressionColumn.this.fireColumnEvent(paramInt1, ExpressionColumn.this.m_cache.getFloat(paramInt1));
        } else if (Double.TYPE == localClass) {
          ExpressionColumn.this.fireColumnEvent(paramInt1, ExpressionColumn.this.m_cache.getDouble(paramInt1));
        } else if (Boolean.TYPE == localClass) {
          ExpressionColumn.this.fireColumnEvent(paramInt1, ExpressionColumn.this.m_cache.getBoolean(paramInt1));
        } else {
          ExpressionColumn.this.fireColumnEvent(paramInt1, ExpressionColumn.this.m_cache.get(paramInt1));
        }
      }
      else
      {
        ExpressionColumn.this.invalidateCache(paramInt1, paramInt2);
        ExpressionColumn.this.fireColumnEvent(0, paramInt1, paramInt2);
      }
    }
    
    public void columnChanged(Column paramColumn, int paramInt, boolean paramBoolean)
    {
      columnChanged(paramInt, paramInt);
    }
    
    public void columnChanged(Column paramColumn, int paramInt, double paramDouble)
    {
      columnChanged(paramInt, paramInt);
    }
    
    public void columnChanged(Column paramColumn, int paramInt, float paramFloat)
    {
      columnChanged(paramInt, paramInt);
    }
    
    public void columnChanged(Column paramColumn, int paramInt1, int paramInt2, int paramInt3)
    {
      columnChanged(paramInt2, paramInt3);
    }
    
    public void columnChanged(Column paramColumn, int paramInt1, int paramInt2)
    {
      columnChanged(paramInt1, paramInt1);
    }
    
    public void columnChanged(Column paramColumn, int paramInt, long paramLong)
    {
      columnChanged(paramInt, paramInt);
    }
    
    public void columnChanged(Column paramColumn, int paramInt, Object paramObject)
    {
      columnChanged(paramInt, paramInt);
    }
    
    public void expressionChanged(Expression paramExpression)
    {
      columnChanged(0, ExpressionColumn.this.m_cache.getRowCount());
      ExpressionColumn.this.init();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\ExpressionColumn.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */