package prefuse.data.column;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.logging.Logger;
import prefuse.data.DataReadOnlyException;
import prefuse.data.DataTypeException;

public class ObjectColumn
  extends AbstractColumn
{
  private Object[] m_values;
  private int m_size;
  
  public ObjectColumn()
  {
    this(Object.class);
  }
  
  public ObjectColumn(Class paramClass)
  {
    this(paramClass, 0, 10, null);
  }
  
  public ObjectColumn(int paramInt)
  {
    this(Object.class, paramInt, paramInt, null);
  }
  
  public ObjectColumn(Class paramClass, int paramInt)
  {
    this(paramClass, paramInt, paramInt, null);
  }
  
  public ObjectColumn(Class paramClass, int paramInt1, int paramInt2, Object paramObject)
  {
    super(paramClass, paramObject);
    if (paramInt2 < paramInt1) {
      throw new IllegalArgumentException("Capacity value can not be less than the row count.");
    }
    this.m_values = new Object[paramInt2];
    try
    {
      Cloneable localCloneable = (Cloneable)paramObject;
      Method localMethod = localCloneable.getClass().getMethod("clone", null);
      for (int i = 0; i < paramInt2; i++) {
        this.m_values[i] = localMethod.invoke(this.m_defaultValue, null);
      }
    }
    catch (Exception localException)
    {
      if (paramObject != null) {
        Logger.getLogger(getClass().getName()).fine("Default value of type \"" + paramObject.getClass().getName() + "\" is not " + "cloneable. Using Object reference directly.");
      }
      Arrays.fill(this.m_values, paramObject);
    }
    this.m_size = paramInt1;
  }
  
  public int getRowCount()
  {
    return this.m_size;
  }
  
  public void setMaximumRow(int paramInt)
  {
    if (paramInt > this.m_values.length)
    {
      int i = Math.max(3 * this.m_values.length / 2 + 1, paramInt);
      Object[] arrayOfObject = new Object[i];
      System.arraycopy(this.m_values, 0, arrayOfObject, 0, this.m_size);
      try
      {
        Cloneable localCloneable = (Cloneable)this.m_defaultValue;
        Method localMethod = localCloneable.getClass().getMethod("clone", null);
        for (int j = this.m_size; j < i; j++) {
          arrayOfObject[j] = localMethod.invoke(this.m_defaultValue, null);
        }
      }
      catch (Exception localException)
      {
        Arrays.fill(arrayOfObject, this.m_size, i, this.m_defaultValue);
      }
      this.m_values = arrayOfObject;
    }
    this.m_size = paramInt;
  }
  
  public void revertToDefault(int paramInt)
  {
    try
    {
      Cloneable localCloneable = (Cloneable)this.m_defaultValue;
      Method localMethod = localCloneable.getClass().getMethod("clone", null);
      set(localMethod.invoke(this.m_defaultValue, null), paramInt);
    }
    catch (Exception localException)
    {
      set(this.m_defaultValue, paramInt);
    }
  }
  
  public Object get(int paramInt)
  {
    if ((paramInt < 0) || (paramInt > this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    return this.m_values[paramInt];
  }
  
  public void set(Object paramObject, int paramInt)
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if ((paramInt < 0) || (paramInt > this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    if ((paramObject == null) || (canSet(paramObject.getClass())))
    {
      Object localObject = this.m_values[paramInt];
      if (localObject == paramObject) {
        return;
      }
      this.m_values[paramInt] = paramObject;
      fireColumnEvent(paramInt, localObject);
    }
    else
    {
      throw new DataTypeException(paramObject.getClass());
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\ObjectColumn.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */