package prefuse.data.expression;

import prefuse.util.collections.CopyOnWriteArrayList;

public abstract class FunctionExpression
  extends AbstractExpression
  implements Function
{
  protected CopyOnWriteArrayList m_params;
  protected final int m_pcount;
  
  protected FunctionExpression(int paramInt)
  {
    this.m_pcount = paramInt;
  }
  
  public abstract String getName();
  
  public void addParameter(Expression paramExpression)
  {
    int i = getParameterCount();
    if ((i != -1) && (paramCount() + 1 > i)) {
      throw new IllegalStateException("This function takes only " + i + " parameters.");
    }
    if (this.m_params == null) {
      this.m_params = new CopyOnWriteArrayList();
    }
    this.m_params.add(paramExpression);
  }
  
  protected int paramCount()
  {
    return this.m_params == null ? 0 : this.m_params.size();
  }
  
  protected final Expression param(int paramInt)
  {
    return (Expression)this.m_params.get(paramInt);
  }
  
  public int getParameterCount()
  {
    return this.m_pcount;
  }
  
  protected void missingParams()
  {
    throw new IllegalStateException("Function is missing parameters: " + getName());
  }
  
  public void visit(ExpressionVisitor paramExpressionVisitor)
  {
    paramExpressionVisitor.visitExpression(this);
    if (paramCount() > 0)
    {
      Object[] arrayOfObject = this.m_params.getArray();
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        paramExpressionVisitor.down();
        ((Expression)arrayOfObject[i]).visit(paramExpressionVisitor);
        paramExpressionVisitor.up();
      }
    }
  }
  
  protected void addChildListeners()
  {
    if (paramCount() > 0)
    {
      Object[] arrayOfObject = this.m_params.getArray();
      for (int i = 0; i < arrayOfObject.length; i++) {
        ((Expression)arrayOfObject[i]).addExpressionListener(this);
      }
    }
  }
  
  protected void removeChildListeners()
  {
    if (paramCount() > 0)
    {
      Object[] arrayOfObject = this.m_params.getArray();
      for (int i = 0; i < arrayOfObject.length; i++) {
        ((Expression)arrayOfObject[i]).removeExpressionListener(this);
      }
    }
  }
  
  public String toString()
  {
    StringBuffer localStringBuffer = new StringBuffer();
    localStringBuffer.append(getName()).append('(');
    for (int i = 0; i < paramCount(); i++)
    {
      if (i > 0) {
        localStringBuffer.append(", ");
      }
      localStringBuffer.append(param(i).toString());
    }
    localStringBuffer.append(')');
    return localStringBuffer.toString();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\expression\FunctionExpression.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */